@@ -4,8 +4,7 @@
  *   SPDX-License-Identifier: GPL-2.0-only OR GPL-3.0-only OR LicenseRef-KDE-Accepted-GPL
  */
 
-#ifndef RATING_H
-#define RATING_H
+#pragma once
 
 #include <QObject>
 #include <QVariant>
@@ -36,13 +35,11 @@ public:
     double sortableRating() const;
 
 private:
-    const QString m_packageName;
-    const quint64 m_ratingCount = 0;
-    const float m_rating = 0;
+    QString m_packageName;
+    quint64 m_ratingCount = 0;
+    float m_rating = 0;
     int m_ratingPoints = 0;
     double m_sortableRating = 0;
 };
 
 Q_DECLARE_METATYPE(Rating)
-
-#endif
