@@ -5,8 +5,7 @@
  *   SPDX-License-Identifier: GPL-2.0-only OR GPL-3.0-only OR LicenseRef-KDE-Accepted-GPL
  */
 
-#ifndef ODRSREVIEWSBACKEND_H
-#define ODRSREVIEWSBACKEND_H
+#pragma once
 
 #include <ReviewsBackend/AbstractReviewsBackend.h>
 #include <ReviewsBackend/ReviewsModel.h>
@@ -26,7 +25,6 @@ public:
     explicit OdrsReviewsBackend();
     ~OdrsReviewsBackend() override;
 
-    QString userName() const override;
     void login() override
     {
     }
@@ -42,6 +40,10 @@ public:
     {
         return false;
     }
+    bool supportsNameChange() const override
+    {
+        return true;
+    }
     void deleteReview(Review *) override
     {
     }
@@ -50,13 +52,21 @@ public:
     {
         return m_isFetching;
     }
-    void submitReview(AbstractResource *, const QString &summary, const QString &description, const QString &rating) override;
     void flagReview(Review *, const QString &, const QString &) override
     {
     }
     void submitUsefulness(Review *review, bool useful) override;
     bool isResourceSupported(AbstractResource *res) const override;
     void emitRatingFetched(AbstractResourcesBackend *backend, const QList<AbstractResource *> &res) const;
+    QString errorMessage() const override
+    {
+        return m_errorMessage;
+    }
+
+    QVector<Rating *> top() const
+    {
+        return m_top;
+    }
 
 private Q_SLOTS:
     void ratingsFetched(KJob *job);
@@ -67,14 +77,21 @@ private Q_SLOTS:
 Q_SIGNALS:
     void ratingsReady();
 
+protected:
+    void sendReview(AbstractResource *app, const QString &summary, const QString &review_text, const QString &rating, const QString &userName) override;
+    QString userName() const override;
+
 private:
+    void fetchRatings();
+    void setFetching(bool);
     QNetworkAccessManager *nam();
     void parseRatings();
     void parseReviews(const QJsonDocument &document, AbstractResource *resource);
 
+    QString m_errorMessage;
     QHash<QString, Rating *> m_ratings;
-    bool m_isFetching;
+    bool m_isFetching = false;
     CachedNetworkAccessManager *m_delayedNam = nullptr;
-};
 
-#endif // ODRSREVIEWSBACKEND_H
+    QVector<Rating *> m_top;
+};
