@@ -24,6 +24,8 @@
 #include <AppStreamQt/icon.h>
 #include <AppStreamQt/metadata.h>
 #include <AppStreamQt/pool.h>
+#include <AppStreamQt/release.h>
+#include <AppStreamQt/version.h>
 
 #include <KAboutData>
 #include <KConfigGroup>
@@ -47,7 +49,9 @@
 #include <QRegularExpression>
 #include <glib.h>
 
+#include <Category/Category.h>
 #include <optional>
+#include <set>
 #include <sys/stat.h>
 
 DISCOVER_BACKEND_PLUGIN(FlatpakBackend)
@@ -91,10 +95,12 @@ public:
         return m_remote && !flatpak_remote_get_disabled(m_remote);
     }
 
-    QString appstreamIconsDir() const {
+    QString appstreamIconsDir() const
+    {
         return m_appstreamIconsDir;
     }
-    QString appstreamDir() const {
+    QString appstreamDir() const
+    {
         Q_ASSERT(m_remote);
         g_autoptr(GFile) appstreamDir = flatpak_remote_get_appstream_dir(m_remote, nullptr);
         if (!appstreamDir) {
@@ -110,6 +116,15 @@ public:
         return m_remote ? QString::fromUtf8(flatpak_remote_get_name(m_remote)) : QString();
     }
 
+    QString title() const
+    {
+        auto ret = m_remote ? QString::fromUtf8(flatpak_remote_get_title(m_remote)) : QString();
+        if (flatpak_installation_get_is_user(m_installation)) {
+            ret = i18nc("user denotes this as user-scoped flatpak repo", "%1 (user)", ret);
+        }
+        return ret;
+    }
+
     FlatpakInstallation *installation() const
     {
         return m_installation;
@@ -124,6 +139,7 @@ public:
 
         m_backend->updateAppState(resource);
 
+        Q_ASSERT(!m_resources.contains(resource->uniqueId()) || m_resources.value(resource->uniqueId()) == resource);
         m_resources.insert(resource->uniqueId(), resource);
         if (!resource->extends().isEmpty()) {
             m_backend->m_extends.append(resource->extends());
@@ -143,19 +159,29 @@ public:
 
     QList<AppStream::Component> componentsByName(const QString &name)
     {
-        QList<AppStream::Component> comps;
-        auto findComponents = [&](const QString &name) {
-            comps = m_pool->componentsById(name);
-            if (!comps.isEmpty())
-                return;
+        QList<AppStream::Component> comps = m_pool->componentsById(name);
+        if (!comps.isEmpty())
+            return comps;
 
-            comps = m_pool->componentsByProvided(AppStream::Provided::KindId, name);
-        };
+        comps = m_pool->componentsByProvided(AppStream::Provided::KindId, name);
+        return comps;
+    }
 
-        findComponents(name);
-        if (comps.isEmpty()) {
-            findComponents(name + QLatin1String(".desktop"));
-        }
+    QList<AppStream::Component> componentsByFlatpakId(const QString &ref)
+    {
+#if ASQ_CHECK_VERSION(0, 16, 0)
+        QList<AppStream::Component> comps = m_pool->componentsByBundleId(AppStream::Bundle::KindFlatpak, ref, false);
+#else
+        QList<AppStream::Component> comps = m_pool->components();
+        comps = kFilter<QList<AppStream::Component>>(comps, [&ref](const AppStream::Component &component) {
+            const QString id = component.bundle(AppStream::Bundle::KindFlatpak).id();
+            return id == ref;
+        });
+#endif
+        if (!comps.isEmpty())
+            return comps;
+
+        comps = m_pool->componentsByProvided(AppStream::Provided::KindId, ref.section('/', 1, 1));
         return comps;
     }
 
@@ -198,14 +224,15 @@ QDebug operator<<(QDebug debug, const FlatpakResource::Id &id)
     return debug;
 }
 
-static FlatpakResource::Id idForRefString(const QStringView &ref)
+static FlatpakResource::Id idForComponent(const AppStream::Component &component)
 {
-    Q_ASSERT(!ref.isEmpty());
-
-    auto parts = ref.split('/');
     // app/app.getspace.Space/x86_64/stable
+    const auto bundleId = component.bundle(AppStream::Bundle::KindFlatpak).id();
+    auto parts = bundleId.splitRef('/');
+    Q_ASSERT(parts.size() == 4);
+
     return {
-        parts[1].toString(),
+        component.id(),
         parts[3].toString(),
         parts[2].toString(),
     };
@@ -261,6 +288,7 @@ FlatpakBackend::FlatpakBackend(QObject *parent)
     , m_updater(new StandardBackendUpdater(this))
     , m_reviews(AppStreamIntegration::global()->reviews())
     , m_cancellable(g_cancellable_new())
+    , m_checkForUpdatesTimer(new QTimer(this))
 {
     g_autoptr(GError) error = nullptr;
 
@@ -282,6 +310,10 @@ FlatpakBackend::FlatpakBackend(QObject *parent)
                                      }));
     });
 
+    m_checkForUpdatesTimer->setInterval(1000);
+    m_checkForUpdatesTimer->setSingleShot(true);
+    connect(m_checkForUpdatesTimer, &QTimer::timeout, this, &FlatpakBackend::checkForUpdates);
+
     /* Override the umask to 022 to make it possible to share files between
      * the plasma-discover process and flatpak system helper process.
      *
@@ -293,13 +325,14 @@ FlatpakBackend::FlatpakBackend(QObject *parent)
 FlatpakBackend::~FlatpakBackend()
 {
     g_cancellable_cancel(m_cancellable);
-    for (auto inst : qAsConst(m_installations))
-        g_object_unref(inst);
     if (!m_threadPool.waitForDone(200)) {
         qDebug() << "could not kill them all" << m_threadPool.activeThreadCount();
     }
     m_threadPool.clear();
 
+    for (auto inst : qAsConst(m_installations))
+        g_object_unref(inst);
+    m_installations.clear();
     g_object_unref(m_cancellable);
 }
 
@@ -432,38 +465,21 @@ FlatpakResource *FlatpakBackend::getAppForInstalledRef(FlatpakInstallation *inst
         if (ret) {
             return ret;
         }
-        ret = source->m_resources.value(idForInstalledRef(ref, QStringLiteral(".desktop")));
-        if (ret) {
-            return ret;
-        }
     }
 
     const QLatin1String name(flatpak_ref_get_name(FLATPAK_REF(ref)));
+    const QLatin1String branch(flatpak_ref_get_branch(FLATPAK_REF(ref)));
     const QString pathExports = FlatpakResource::installationPath(installation) + QLatin1String("/exports/");
     const QString pathApps = pathExports + QLatin1String("share/applications/");
+    const QString refId = refToBundleId(FLATPAK_REF(ref));
     AppStream::Component cid;
     if (source && source->m_pool) {
-        QList<AppStream::Component> comps = source->componentsByName(name);
-        if (comps.isEmpty()) {
-            const QString bundleId = refToBundleId(FLATPAK_REF(ref));
-            comps = kFilter<QList<AppStream::Component>>(comps, [&bundleId](const AppStream::Component &comp) -> bool {
-                return comp.bundle(AppStream::Bundle::Kind::KindFlatpak).id() == bundleId;
-            });
-        }
-
+        QList<AppStream::Component> comps = source->componentsByFlatpakId(refId);
         if (comps.isEmpty()) {
             g_autoptr(GBytes) metadata = flatpak_installed_ref_load_appdata(ref, 0, 0);
             if (metadata) {
                 auto meta = metadataFromBytes(metadata, m_cancellable);
-                const auto componentsProvided = meta->components();
-                if (!componentsProvided.isEmpty() && name != componentsProvided.constFirst().id()) {
-                    qDebug() << "mismatch between flatpak and appstream" << name << componentsProvided.constFirst().id();
-                    comps = source->m_pool->componentsById(componentsProvided.constFirst().id());
-                }
-
-                if (comps.isEmpty()) {
-                    comps = componentsProvided;
-                }
+                comps = meta->components();
             }
         }
 
@@ -481,7 +497,7 @@ FlatpakResource *FlatpakBackend::getAppForInstalledRef(FlatpakInstallation *inst
             if (QFile::exists(fnDesktop))
                 qDebug() << "Failed to parse appstream metadata:" << error << fnDesktop;
 
-            cid.setId(QString::fromLatin1(flatpak_ref_get_name(FLATPAK_REF(ref))));
+            cid.setId(name);
 #if FLATPAK_CHECK_VERSION(1, 1, 2)
             cid.setName(QString::fromUtf8(flatpak_installed_ref_get_appdata_name(ref)));
 #endif
@@ -489,8 +505,27 @@ FlatpakResource *FlatpakBackend::getAppForInstalledRef(FlatpakInstallation *inst
             cid = metadata.component();
     }
 
+    if (cid.bundle(AppStream::Bundle::KindFlatpak).isEmpty()) {
+        AppStream::Bundle b;
+        b.setKind(AppStream::Bundle::KindFlatpak);
+        b.setId(refId);
+        cid.addBundle(b);
+    }
+
+    if (source && cid.isValid()) {
+        auto ret = source->m_resources.value(idForComponent(cid));
+        if (ret) {
+            return ret;
+        }
+    }
+
+    if (!source) {
+        return nullptr;
+    }
+
     FlatpakResource *resource = new FlatpakResource(cid, source->installation(), const_cast<FlatpakBackend *>(this));
     resource->setOrigin(source->name());
+    resource->setDisplayOrigin(source->title());
     resource->setIconPath(pathExports);
     resource->updateFromRef(FLATPAK_REF(ref));
     resource->setState(AbstractResource::Installed);
@@ -517,7 +552,7 @@ QSharedPointer<FlatpakSource> FlatpakBackend::findSource(FlatpakInstallation *in
     }
 
     qWarning() << "Could not find source:" << installation << origin;
-    Q_UNREACHABLE();
+    return {};
 }
 
 FlatpakResource *FlatpakBackend::getRuntimeForApp(FlatpakResource *resource) const
@@ -620,6 +655,24 @@ void FlatpakBackend::addAppFromFlatpakBundle(const QUrl &url, ResultsStream *str
         tempFile.remove();
     }
 
+    g_autoptr(GPtrArray) refs = flatpak_installation_list_installed_refs(preferredInstallation(), m_cancellable, &localError);
+    if (!refs) {
+        qWarning() << "Failed to get list of installed refs for listing local updates:" << localError->message;
+        return;
+    }
+
+    for (uint i = 0; i < refs->len; i++) {
+        FlatpakRef *ref = FLATPAK_REF(g_ptr_array_index(refs, i));
+        FlatpakInstalledRef *iref = FLATPAK_INSTALLED_REF(g_ptr_array_index(refs, i));
+        if (qstrcmp(flatpak_ref_get_commit(ref), flatpak_ref_get_commit(FLATPAK_REF(bundleRef))) == 0) {
+            auto res = getAppForInstalledRef(preferredInstallation(), iref, nullptr);
+            if (res) {
+                Q_EMIT stream->resourcesFound({res});
+            }
+            return;
+        }
+    }
+
     FlatpakResource *resource = new FlatpakResource(asComponent, preferredInstallation(), this);
     if (!updateAppMetadata(resource, metadataContent)) {
         delete resource;
@@ -642,7 +695,7 @@ void FlatpakBackend::addAppFromFlatpakBundle(const QUrl &url, ResultsStream *str
     }
 
     const QString origin = QString::fromUtf8(flatpak_bundle_ref_get_origin(bundleRef));
-
+    resource->updateFromRef(FLATPAK_REF(bundleRef));
     resource->setDownloadSize(0);
     resource->setInstalledSize(flatpak_bundle_ref_get_installed_size(bundleRef));
     resource->setPropertyState(FlatpakResource::DownloadSize, FlatpakResource::AlreadyKnown);
@@ -651,20 +704,26 @@ void FlatpakBackend::addAppFromFlatpakBundle(const QUrl &url, ResultsStream *str
     resource->setOrigin(origin.isEmpty() ? i18n("Local bundle") : origin);
     resource->setResourceFile(url);
     resource->setState(FlatpakResource::None);
-    resource->setType(FlatpakResource::DesktopApp);
 
     if (!m_localSource) {
         m_localSource.reset(new FlatpakSource(this, preferredInstallation()));
         m_flatpakSources += m_localSource;
     }
     m_localSource->addResource(resource);
-    stream->resourcesFound({resource});
+    Q_EMIT stream->resourcesFound({resource});
+}
+
+QString composeRef(bool isRuntime, const QString &name, const QString &branch)
+{
+    return (isRuntime ? "runtime/" : "app/") + name + '/' + flatpak_get_default_arch() + '/' + branch;
 }
 
 AppStream::Component fetchComponentFromRemote(const QSettings &settings, GCancellable *cancellable)
 {
     const QString name = settings.value(QStringLiteral("Flatpak Ref/Name")).toString();
+    const QString branch = settings.value(QStringLiteral("Flatpak Ref/Branch")).toString();
     const QString remoteName = settings.value(QStringLiteral("Flatpak Ref/SuggestRemoteName")).toString();
+    const bool isRuntime = settings.value(QStringLiteral("Flatpak Ref/IsRuntime")).toBool();
 
     AppStream::Component asComponent;
     asComponent.addUrl(AppStream::Component::UrlKindHomepage, settings.value(QStringLiteral("Flatpak Ref/Homepage")).toString());
@@ -675,7 +734,7 @@ AppStream::Component fetchComponentFromRemote(const QSettings &settings, GCancel
 
     AppStream::Bundle b;
     b.setKind(AppStream::Bundle::KindFlatpak);
-    b.setId(asComponent.name());
+    b.setId(composeRef(isRuntime, asComponent.name(), branch));
     asComponent.addBundle(b);
 
     // We are going to create a temporary installation and add the remote to it.
@@ -735,15 +794,13 @@ AppStream::Component fetchComponentFromRemote(const QSettings &settings, GCancel
         return {asComponent};
     }
 
-    const QString branch = settings.value(QStringLiteral("Flatpak Ref/Branch")).toString();
-
-    auto comps = pool.componentsById(name);
-    if (!branch.isEmpty()) {
-        const QString suffix = QLatin1Char('/') + branch;
-        comps = kFilter<QList<AppStream::Component>>(comps, [&suffix](const AppStream::Component &component) {
-            return component.bundle(AppStream::Bundle::KindFlatpak).id().endsWith(suffix);
-        });
-    }
+    // TODO optimise, this lookup should happen in libappstream
+    auto comps = pool.components();
+    comps = kFilter<QList<AppStream::Component>>(comps, [name, branch](const AppStream::Component &component) {
+        const QString id = component.bundle(AppStream::Bundle::KindFlatpak).id();
+        // app/app.getspace.Space/x86_64/stable
+        return id.section(QLatin1Char('/'), 1, 1) == name && (branch.isEmpty() || id.section(QLatin1Char('/'), 3, 3) == branch);
+    });
     if (comps.isEmpty()) {
         qDebug() << "could not find" << name << "in" << remoteName;
         return asComponent;
@@ -758,6 +815,8 @@ void FlatpakBackend::addAppFromFlatpakRef(const QUrl &url, ResultsStream *stream
     const QString refurl = settings.value(QStringLiteral("Flatpak Ref/Url")).toString();
     const QString name = settings.value(QStringLiteral("Flatpak Ref/Name")).toString();
     const QString remoteName = settings.value(QStringLiteral("Flatpak Ref/SuggestRemoteName")).toString();
+    const QString branch = settings.value(QStringLiteral("Flatpak Ref/Branch")).toString();
+    const bool isRuntime = settings.value(QStringLiteral("Flatpak Ref/IsRuntime")).toBool();
     g_autoptr(GError) error = nullptr;
 
     // If we already added the remote, just go with it
@@ -770,12 +829,13 @@ void FlatpakBackend::addAppFromFlatpakRef(const QUrl &url, ResultsStream *stream
         m_refreshAppstreamMetadataJobs.insert(remote);
         auto source = integrateRemote(preferredInstallation(), remote);
         if (source) {
-            auto searchComponent = [this, stream, source, name] {
-                auto comps = source->componentsByName(name);
+            const QString ref = composeRef(isRuntime, name, branch);
+            auto searchComponent = [this, stream, source, ref] {
+                auto comps = source->componentsByFlatpakId(ref);
                 auto resources = kTransform<QVector<AbstractResource *>>(comps, [this, source](const auto &comp) {
                     return resourceForComponent(comp, source);
                 });
-                stream->resourcesFound(resources);
+                Q_EMIT stream->resourcesFound(resources);
                 stream->finish();
             };
             if (source->m_pool) {
@@ -801,10 +861,11 @@ void FlatpakBackend::addAppFromFlatpakRef(const QUrl &url, ResultsStream *stream
     resource->setResourceFile(url);
     resource->setResourceLocation(QUrl(refurl));
     resource->setOrigin(remoteName);
+    resource->setDisplayOrigin(remote ? QString::fromUtf8(flatpak_remote_get_title(remote)) : QString());
     resource->setFlatpakName(name);
     resource->setArch(flatpak_get_default_arch());
-    resource->setBranch(settings.value(QStringLiteral("Flatpak Ref/Branch")).toString());
-    resource->setType(settings.value(QStringLiteral("Flatpak Ref/IsRuntime")).toBool() ? FlatpakResource::Runtime : FlatpakResource::DesktopApp);
+    resource->setBranch(branch);
+    resource->setType(isRuntime ? FlatpakResource::Runtime : FlatpakResource::DesktopApp);
 
     QUrl runtimeUrl = QUrl(settings.value(QStringLiteral("Flatpak Ref/RuntimeRepo")).toString());
     auto refSource = QSharedPointer<FlatpakSource>::create(this, preferredInstallation());
@@ -818,7 +879,7 @@ void FlatpakBackend::addAppFromFlatpakRef(const QUrl &url, ResultsStream *stream
             const auto metadata = fw->result();
             // Even when we failed to fetch information about runtime we still want to show the application
             if (metadata.isEmpty()) {
-                Q_EMIT onFetchMetadataFinished(resource, metadata);
+                onFetchMetadataFinished(resource, metadata);
             } else {
                 updateAppMetadata(resource, metadata);
 
@@ -833,7 +894,7 @@ void FlatpakBackend::addAppFromFlatpakRef(const QUrl &url, ResultsStream *stream
                                     installApplication(res);
                                 }
                                 refSource->addResource(resource);
-                                stream->resourcesFound({resource});
+                                Q_EMIT stream->resourcesFound({resource});
                                 stream->finish();
                             });
 
@@ -844,16 +905,15 @@ void FlatpakBackend::addAppFromFlatpakRef(const QUrl &url, ResultsStream *stream
                     refSource->addResource(resource);
                 }
             }
-            stream->resourcesFound({resource});
+            Q_EMIT stream->resourcesFound({resource});
             stream->finish();
         });
         fw->setFuture(QtConcurrent::run(&m_threadPool, &FlatpakRunnables::fetchMetadata, resource, m_cancellable));
     } else {
         refSource->addResource(resource);
-        stream->resourcesFound({resource});
+        Q_EMIT stream->resourcesFound({resource});
         stream->finish();
     }
-
 }
 
 void FlatpakBackend::addSourceFromFlatpakRepo(const QUrl &url, ResultsStream *stream)
@@ -881,7 +941,7 @@ void FlatpakBackend::addSourceFromFlatpakRepo(const QUrl &url, ResultsStream *st
     asComponent.setSummary(settings.value(QStringLiteral("Flatpak Repo/Comment")).toString());
     asComponent.setDescription(settings.value(QStringLiteral("Flatpak Repo/Description")).toString());
     asComponent.setName(title);
-    asComponent.setId(settings.value(QStringLiteral("Flatpak Ref/Name")).toString());
+    asComponent.setId(settings.value(QStringLiteral("Flatpak Repo/Title")).toString());
 
     const QString iconUrl = settings.value(QStringLiteral("Flatpak Repo/Icon")).toString();
     if (!iconUrl.isEmpty()) {
@@ -907,7 +967,7 @@ void FlatpakBackend::addSourceFromFlatpakRepo(const QUrl &url, ResultsStream *st
         resource->setState(AbstractResource::State::Installed);
     }
 
-    stream->resourcesFound({resource});
+    Q_EMIT stream->resourcesFound({resource});
 }
 
 void FlatpakBackend::loadAppsFromAppstreamData()
@@ -950,9 +1010,13 @@ void FlatpakBackend::loadRemote(FlatpakInstallation *installation, FlatpakRemote
 
     g_autofree char *path_str = g_file_get_path(fileTimestamp);
     QFileInfo fileInfo(QFile::encodeName(path_str));
-    // Refresh appstream metadata in case they have never been refreshed or the cache is older than 6 hours
-    if (!fileInfo.exists() || fileInfo.lastModified().toUTC().secsTo(QDateTime::currentDateTimeUtc()) > 21600) {
-        checkForUpdates(installation, remote);
+    if (!fileInfo.exists()) {
+        checkForRemoteUpdates(installation, remote);
+
+        // Refresh appstream metadata in case they have never been refreshed or the cache is older than 6 hours
+        if (fileInfo.lastModified().toUTC().secsTo(QDateTime::currentDateTimeUtc()) > 21600) {
+            connect(this, &FlatpakBackend::initialized, m_checkForUpdatesTimer, qOverload<>(&QTimer::start));
+        }
     } else {
         auto source = integrateRemote(installation, remote);
         Q_ASSERT(findSource(installation, flatpak_remote_get_name(remote)) == source);
@@ -1068,7 +1132,7 @@ QSharedPointer<FlatpakSource> FlatpakBackend::integrateRemote(FlatpakInstallatio
 void FlatpakBackend::loadLocalUpdates(FlatpakInstallation *flatpakInstallation)
 {
     g_autoptr(GError) localError = nullptr;
-    g_autoptr(GPtrArray) refs = flatpak_installation_list_installed_refs_for_update(flatpakInstallation, m_cancellable, &localError);
+    g_autoptr(GPtrArray) refs = flatpak_installation_list_installed_refs(flatpakInstallation, m_cancellable, &localError);
     if (!refs) {
         qWarning() << "Failed to get list of installed refs for listing local updates:" << localError->message;
         return;
@@ -1094,6 +1158,7 @@ void FlatpakBackend::loadLocalUpdates(FlatpakInstallation *flatpakInstallation)
             resource->setState(AbstractResource::Upgradeable);
             updateAppSize(resource);
         }
+        Q_ASSERT(!resource->temporarySource());
     }
 }
 
@@ -1118,7 +1183,7 @@ bool FlatpakBackend::setupFlatpakInstallations(GError **error)
         qDebug() << "running flatpak backend on test mode" << path;
         g_autoptr(GFile) file = g_file_new_for_path(QFile::encodeName(path).constData());
         m_installations << flatpak_installation_new_for_path(file, true, m_cancellable, error);
-        return true;
+        return m_installations.constLast() != nullptr;
     }
 
     g_autoptr(GPtrArray) installations = flatpak_get_system_installations(m_cancellable, error);
@@ -1381,16 +1446,15 @@ ResultsStream *FlatpakBackend::search(const AbstractResourcesBackend::Filters &f
                     for (auto ref : qAsConst(it.value())) {
                         bool fresh;
                         auto resource = getAppForInstalledRef(it.key(), ref, &fresh);
-#if FLATPAK_CHECK_VERSION(1, 1, 2)
-                        resource->setAvailableVersion(QString::fromUtf8(flatpak_installed_ref_get_appdata_version(ref)));
-#endif
                         g_object_unref(ref);
-                        resource->setState(AbstractResource::Upgradeable, !fresh);
-                        updateAppSize(resource);
-                        if (resource->resourceType() == FlatpakResource::Runtime) {
-                            resources.prepend(resource);
-                        } else {
-                            resources.append(resource);
+                        if (resource) {
+                            resource->setState(AbstractResource::Upgradeable, !fresh);
+                            updateAppSize(resource);
+                            if (resource->resourceType() == FlatpakResource::Runtime) {
+                                resources.prepend(resource);
+                            } else {
+                                resources.append(resource);
+                            }
                         }
                     }
                 }
@@ -1405,8 +1469,12 @@ ResultsStream *FlatpakBackend::search(const AbstractResourcesBackend::Filters &f
             auto cancellable = m_cancellable;
             fw->setFuture(QtConcurrent::run(&m_threadPool, [installations, cancellable] {
                 QHash<FlatpakInstallation *, QVector<FlatpakInstalledRef *>> ret;
+                if (g_cancellable_is_cancelled(cancellable)) {
+                    qWarning() << "Job cancelled";
+                    return ret;
+                }
 
-                for (auto installation : installations) {
+                for (auto installation : std::as_const(installations)) {
                     g_autoptr(GError) localError = nullptr;
                     g_autoptr(GPtrArray) refs = flatpak_installation_list_installed_refs_for_update(installation, cancellable, &localError);
                     if (!refs) {
@@ -1415,7 +1483,12 @@ ResultsStream *FlatpakBackend::search(const AbstractResourcesBackend::Filters &f
                     }
                     if (g_cancellable_is_cancelled(cancellable)) {
                         qWarning() << "Job cancelled";
-                        return ret;
+                        ret.clear();
+                        break;
+                    }
+
+                    if (refs->len == 0) {
+                        continue;
                     }
 
                     auto &current = ret[installation];
@@ -1440,7 +1513,7 @@ ResultsStream *FlatpakBackend::search(const AbstractResourcesBackend::Filters &f
         auto stream = new ResultsStream(QStringLiteral("FlatpakStream-installed"));
         auto f = [this, stream, filter] {
             QVector<AbstractResource *> resources;
-            for (auto installation : m_installations) {
+            for (auto installation : std::as_const(m_installations)) {
                 g_autoptr(GError) localError = nullptr;
                 g_autoptr(GPtrArray) refs = flatpak_installation_list_installed_refs(installation, m_cancellable, &localError);
                 if (!refs) {
@@ -1457,6 +1530,9 @@ ResultsStream *FlatpakBackend::search(const AbstractResourcesBackend::Filters &f
                         continue;
 
                     auto resource = getAppForInstalledRef(installation, ref);
+                    if (!resource) {
+                        continue;
+                    }
                     if (!filter.search.isEmpty() && !resource->name().contains(filter.search, Qt::CaseInsensitive))
                         continue;
 
@@ -1484,16 +1560,21 @@ ResultsStream *FlatpakBackend::search(const AbstractResourcesBackend::Filters &f
     auto f = [this, stream, filter]() {
         QVector<AbstractResource *> prioritary, rest;
         for (const auto &source : qAsConst(m_flatpakSources)) {
-            QVector<FlatpakResource *> resources;
+            QList<FlatpakResource *> resources;
             if (source->m_pool) {
-                resources = kTransform<QVector<FlatpakResource *>>(source->m_pool->search(filter.search), [this, &source](const auto &comp) {
+                const auto a = !filter.search.isEmpty() ? source->m_pool->search(filter.search)
+#if ASQ_CHECK_VERSION(0, 15, 6)
+                    : filter.category ? AppStreamUtils::componentsByCategories(source->m_pool, filter.category, AppStream::Bundle::KindFlatpak)
+#endif
+                                      : source->m_pool->components();
+                resources = kTransform<QList<FlatpakResource *>>(a, [this, &source](const auto &comp) {
                     return resourceForComponent(comp, source);
                 });
             } else {
-                resources = source->m_resources.values().toVector();
+                resources = source->m_resources.values();
             }
 
-            for (auto r : resources) {
+            for (auto r : std::as_const(resources)) {
                 const bool matchById = r->appstreamId().compare(filter.search, Qt::CaseInsensitive) == 0;
                 if (r->type() == AbstractResource::Technical && filter.state != AbstractResource::Upgradeable && !matchById) {
                     continue;
@@ -1570,13 +1651,21 @@ ResultsStream *FlatpakBackend::findResourceByPackageName(const QUrl &url)
         if (appstreamIds.isEmpty())
             Q_EMIT passiveMessage(i18n("Malformed appstream url '%1'", url.toDisplayString()));
         else {
-            auto stream = new ResultsStream(QStringLiteral("FlatpakStream"));
-            auto f = [this, stream, appstreamIds]() {
-                const auto resources = kAppend<QVector<AbstractResource *>>(appstreamIds, [this](const QString &appstreamId) {
-                    return resourcesByAppstreamName(appstreamId);
-                });
-                if (!resources.isEmpty())
-                    Q_EMIT stream->resourcesFound(resources);
+            auto stream = new ResultsStream(QStringLiteral("FlatpakStream-AppStreamUrl"));
+            auto f = [this, stream, appstreamIds] {
+                std::set<AbstractResource *> resources;
+                QVector<AbstractResource *> resourcesVector;
+                for (const auto &appstreamId : appstreamIds) {
+                    const auto resourcesFound = resourcesByAppstreamName(appstreamId);
+                    for (auto res : resourcesFound) {
+                        auto [x, inserted] = resources.insert(res);
+                        if (inserted) {
+                            resourcesVector.append(res);
+                        }
+                    }
+                }
+                if (!resourcesVector.isEmpty())
+                    Q_EMIT stream->resourcesFound(resourcesVector);
                 stream->finish();
             };
 
@@ -1593,16 +1682,19 @@ ResultsStream *FlatpakBackend::findResourceByPackageName(const QUrl &url)
 
 FlatpakResource *FlatpakBackend::resourceForComponent(const AppStream::Component &component, const QSharedPointer<FlatpakSource> &source) const
 {
-    auto resource = source->m_resources.value(idForRefString(component.bundle(AppStream::Bundle::KindFlatpak).id()));
+    const auto ref = idForComponent(component);
+    auto resource = source->m_resources.value(ref);
     if (resource) {
         return resource;
     }
 
     FlatpakResource *res = new FlatpakResource(component, source->installation(), const_cast<FlatpakBackend *>(this));
     res->setOrigin(source->name());
+    res->setDisplayOrigin(source->title());
     res->setIconPath(source->appstreamIconsDir());
     res->updateFromAppStream();
     source->addResource(res);
+    Q_ASSERT(ref == res->uniqueId());
     return res;
 }
 
@@ -1619,7 +1711,7 @@ AbstractReviewsBackend *FlatpakBackend::reviewsBackend() const
 void FlatpakBackend::checkRepositories(const QMap<QString, QStringList> &names)
 {
     auto flatpakInstallationByPath = [this](const QString &path) -> FlatpakInstallation * {
-        for (auto inst : m_installations) {
+        for (auto inst : std::as_const(m_installations)) {
             if (FlatpakResource::installationPath(inst) == path)
                 return inst;
         }
@@ -1692,6 +1784,10 @@ Transaction *FlatpakBackend::installApplication(AbstractResource *app, const Add
         if (status == Transaction::Status::DoneStatus) {
             if (auto tempSource = resource->temporarySource()) {
                 auto source = findSource(resource->installation(), resource->origin());
+                if (!source) {
+                    // It could mean that it's still integrating after checkRepositories It should update itself
+                    return;
+                }
                 resource->setTemporarySource({});
                 const auto id = resource->uniqueId();
                 source->m_resources.insert(id, resource);
@@ -1737,16 +1833,16 @@ Transaction *FlatpakBackend::removeApplication(AbstractResource *app)
 
 void FlatpakBackend::checkForUpdates()
 {
-    for (auto source : qAsConst(m_flatpakSources)) {
+    for (const auto &source : qAsConst(m_flatpakSources)) {
         if (source->remote()) {
             Q_ASSERT(!m_refreshAppstreamMetadataJobs.contains(source->remote()));
             m_refreshAppstreamMetadataJobs.insert(source->remote());
-            checkForUpdates(source->installation(), source->remote());
+            checkForRemoteUpdates(source->installation(), source->remote());
         }
     }
 }
 
-void FlatpakBackend::checkForUpdates(FlatpakInstallation *installation, FlatpakRemote *remote)
+void FlatpakBackend::checkForRemoteUpdates(FlatpakInstallation *installation, FlatpakRemote *remote)
 {
     Q_ASSERT(remote);
     const bool needsIntegration = m_refreshAppstreamMetadataJobs.contains(remote);
@@ -1774,14 +1870,14 @@ QString FlatpakBackend::displayName() const
     return QStringLiteral("Flatpak");
 }
 
-HelpfulError *FlatpakBackend::explainDysfunction() const
+InlineMessage *FlatpakBackend::explainDysfunction() const
 {
     if (m_flatpakSources.isEmpty()) {
-        return new HelpfulError(QStringLiteral("emblem-error"), i18n("There are no Flatpak sources."), m_sources->actions());
+        return new InlineMessage(InlineMessage::Error, QStringLiteral("emblem-error"), i18n("There are no Flatpak sources."), m_sources->actions());
     }
     for (const auto &source : m_flatpakSources) {
         if (source->m_pool && !source->m_pool->lastError().isEmpty()) {
-            return new HelpfulError(QStringLiteral("emblem-error"), i18n("Failed to load \"%1\" source", source->name()));
+            return new InlineMessage(InlineMessage::Error, QStringLiteral("emblem-error"), i18n("Failed to load \"%1\" source", source->name()));
         }
     }
     return AbstractResourcesBackend::explainDysfunction();
