@@ -10,9 +10,11 @@
 #include <glib.h>
 
 #include <QDebug>
+#include <QFutureWatcher>
 #include <QTimer>
 #include <QtConcurrentRun>
-#include <QFutureWatcher>
+
+using namespace std::chrono_literals;
 
 static void installationChanged(GFileMonitor *monitor, GFile *child, GFile *other_file, GFileMonitorEvent event_type, gpointer self)
 {
@@ -21,7 +23,7 @@ static void installationChanged(GFileMonitor *monitor, GFile *child, GFile *othe
     Q_UNUSED(other_file);
     Q_UNUSED(event_type);
 
-    FlatpakNotifier::Installation *installation = (FlatpakNotifier::Installation*) self;
+    FlatpakNotifier::Installation *installation = (FlatpakNotifier::Installation *)self;
     if (!installation)
         return;
 
@@ -29,14 +31,14 @@ static void installationChanged(GFileMonitor *monitor, GFile *child, GFile *othe
     notifier->loadRemoteUpdates(installation);
 }
 
-FlatpakNotifier::FlatpakNotifier(QObject* parent)
+FlatpakNotifier::FlatpakNotifier(QObject *parent)
     : BackendNotifierModule(parent)
     , m_user(this)
     , m_system(this)
     , m_cancellable(g_cancellable_new())
 {
     QTimer *dailyCheck = new QTimer(this);
-    dailyCheck->setInterval(24 * 60 * 60 * 1000); //refresh at least once every day
+    dailyCheck->setInterval(24h); // refresh at least once every day
     connect(dailyCheck, &QTimer::timeout, this, &FlatpakNotifier::recheckSystemUpdateNeeded);
 }
 
@@ -72,48 +74,48 @@ void FlatpakNotifier::recheckSystemUpdateNeeded()
     }
 }
 
-void FlatpakNotifier::onFetchUpdatesFinished(Installation *installation, GPtrArray *updates)
+void FlatpakNotifier::onFetchUpdatesFinished(Installation *installation, bool hasUpdates)
 {
-    bool hasUpdates = false;
-
-    g_autoptr(GPtrArray) fetchedUpdates = updates;
-
-    for (uint i = 0; !hasUpdates && i < fetchedUpdates->len; i++) {
-        FlatpakInstalledRef *ref = FLATPAK_INSTALLED_REF(g_ptr_array_index(fetchedUpdates, i));
-        const QString refName = QString::fromUtf8(flatpak_ref_get_name(FLATPAK_REF(ref)));
-        // FIXME right now I can't think of any other filter than this, in FlatpakBackend updates are matched
-        // with apps so .Locale/.Debug subrefs are not shown and updated automatically. Also this will show
-        // updates for refs we don't show in Discover if appstream metadata or desktop file for them is not found
-        if (refName.endsWith(QLatin1String(".Locale")) || refName.endsWith(QLatin1String(".Debug"))) {
-            continue;
-        }
-        hasUpdates = true;
+    if (installation->m_hasUpdates == hasUpdates) {
+        return;
     }
-    const bool changed = this->hasUpdates() != hasUpdates;
+    bool hadUpdates = this->hasUpdates();
     installation->m_hasUpdates = hasUpdates;
 
-    if (changed) {
+    if (hadUpdates != this->hasUpdates()) {
         Q_EMIT foundUpdates();
     }
 }
 
-void FlatpakNotifier::loadRemoteUpdates(Installation* installation)
+void FlatpakNotifier::loadRemoteUpdates(Installation *installation)
 {
-    auto fw = new QFutureWatcher<GPtrArray *>(this);
-    connect(fw, &QFutureWatcher<GPtrArray *>::finished, this, [this, installation, fw](){
-        g_autoptr(GPtrArray) refs = fw->result();
-        if (refs)
-            onFetchUpdatesFinished(installation, refs);
+    auto fw = new QFutureWatcher<bool>(this);
+    connect(fw, &QFutureWatcher<bool>::finished, this, [this, installation, fw]() {
+        onFetchUpdatesFinished(installation, fw->result());
         fw->deleteLater();
     });
-    fw->setFuture(QtConcurrent::run( [installation]() -> GPtrArray * {
+    fw->setFuture(QtConcurrent::run([installation]() -> bool {
         g_autoptr(GCancellable) cancellable = g_cancellable_new();
         g_autoptr(GError) localError = nullptr;
-        GPtrArray *refs = flatpak_installation_list_installed_refs_for_update(installation->m_installation, cancellable, &localError);
-        if (!refs) {
+        g_autoptr(GPtrArray) fetchedUpdates = flatpak_installation_list_installed_refs_for_update(installation->m_installation, cancellable, &localError);
+        bool hasUpdates = false;
+
+        if (!fetchedUpdates) {
             qWarning() << "Failed to get list of installed refs for listing updates: " << localError->message;
+            return false;
         }
-        return refs;
+        for (uint i = 0; !hasUpdates && i < fetchedUpdates->len; i++) {
+            FlatpakInstalledRef *ref = FLATPAK_INSTALLED_REF(g_ptr_array_index(fetchedUpdates, i));
+            const QString refName = QString::fromUtf8(flatpak_ref_get_name(FLATPAK_REF(ref)));
+            // FIXME right now I can't think of any other filter than this, in FlatpakBackend updates are matched
+            // with apps so .Locale/.Debug subrefs are not shown and updated automatically. Also this will show
+            // updates for refs we don't show in Discover if appstream metadata or desktop file for them is not found
+            if (refName.endsWith(QLatin1String(".Locale")) || refName.endsWith(QLatin1String(".Debug"))) {
+                continue;
+            }
+            hasUpdates = true;
+        }
+        return hasUpdates;
     }));
 }
 
@@ -122,7 +124,7 @@ bool FlatpakNotifier::hasUpdates()
     return m_system.m_hasUpdates || m_user.m_hasUpdates;
 }
 
-bool FlatpakNotifier::Installation::ensureInitialized(std::function<FlatpakInstallation*()> func, GCancellable *cancellable, GError **error)
+bool FlatpakNotifier::Installation::ensureInitialized(std::function<FlatpakInstallation *()> func, GCancellable *cancellable, GError **error)
 {
     if (!m_installation) {
         m_installation = func();
@@ -134,9 +136,19 @@ bool FlatpakNotifier::Installation::ensureInitialized(std::function<FlatpakInsta
 
 bool FlatpakNotifier::setupFlatpakInstallations(GError **error)
 {
-    if (!m_system.ensureInitialized([this, error] { return flatpak_installation_new_system(m_cancellable, error); }, m_cancellable, error))
+    if (!m_system.ensureInitialized(
+            [this, error] {
+                return flatpak_installation_new_system(m_cancellable, error);
+            },
+            m_cancellable,
+            error))
         return false;
-    if (!m_user.ensureInitialized([this, error] { return flatpak_installation_new_system(m_cancellable, error); }, m_cancellable, error))
+    if (!m_user.ensureInitialized(
+            [this, error] {
+                return flatpak_installation_new_user(m_cancellable, error);
+            },
+            m_cancellable,
+            error))
         return false;
 
     return true;
