@@ -4,13 +4,14 @@
  *   SPDX-License-Identifier: GPL-2.0-only OR GPL-3.0-only OR LicenseRef-KDE-Accepted-GPL
  */
 
-#ifndef FLATPAKTRANSACTIONTHREAD_H
-#define FLATPAKTRANSACTIONTHREAD_H
+#pragma once
 
 #include "flatpak-helper.h"
 #include <gio/gio.h>
 #include <glib.h>
 
+#include <QMap>
+#include <QStringList>
 #include <QThread>
 #include <Transaction/Transaction.h>
 
@@ -34,13 +35,10 @@ public:
 
     QString errorMessage() const;
     bool result() const;
-    bool cancelled() const
-    {
-        return m_cancelled;
-    }
+    bool cancelled() const;
 
     void addErrorMessage(const QString &error);
-    QStringList addedRepositories() const
+    QMap<QString, QStringList> addedRepositories() const
     {
         return m_addedRepositories;
     }
@@ -49,21 +47,28 @@ Q_SIGNALS:
     void progressChanged(int progress);
     void speedChanged(quint64 speed);
     void passiveMessage(const QString &msg);
+    void webflowStarted(const QUrl &url, int id);
+    void webflowDone(int id);
 
 private:
     static gboolean
     add_new_remote_cb(FlatpakTransaction * /*object*/, gint /*reason*/, gchar *from_id, gchar *suggested_remote_name, gchar *url, gpointer user_data);
+    static void progress_changed_cb(FlatpakTransactionProgress *progress, gpointer user_data);
+    static void
+    new_operation_cb(FlatpakTransaction * /*object*/, FlatpakTransactionOperation *operation, FlatpakTransactionProgress *progress, gpointer user_data);
+
+    static gboolean webflowStart(FlatpakTransaction *transaction, const char *remote, const char *url, GVariant *options, guint id, gpointer user_data);
+    static void webflowDoneCallback(FlatpakTransaction *transaction, GVariant *options, guint id, gpointer user_data);
 
     FlatpakTransaction *m_transaction;
     bool m_result = false;
-    bool m_cancelled = false;
     int m_progress = 0;
     quint64 m_speed = 0;
     QString m_errorMessage;
     GCancellable *m_cancellable;
     FlatpakResource *const m_app;
     const Transaction::Role m_role;
-    QStringList m_addedRepositories;
-};
+    QMap<QString, QStringList> m_addedRepositories;
 
-#endif // FLATPAKTRANSACTIONJOB_H
+    QVector<int> m_webflows;
+};
