@@ -6,6 +6,7 @@
  */
 
 #include "FwupdBackend.h"
+#include "../DiscoverVersion.h"
 #include "FwupdResource.h"
 #include "FwupdSourcesBackend.h"
 #include "FwupdTransaction.h"
@@ -28,7 +29,13 @@ FwupdBackend::FwupdBackend(QObject *parent)
     , m_updater(new StandardBackendUpdater(this))
     , m_cancellable(g_cancellable_new())
 {
-    fwupd_client_set_user_agent_for_package(client, "plasma-discover", "1.0");
+    g_autoptr(GError) error = nullptr;
+    if (!fwupd_client_connect(client, m_cancellable, &error)) {
+        handleError(error);
+        m_isValid = false;
+        return;
+    }
+    fwupd_client_set_user_agent_for_package(client, "plasma-discover", version.data());
     connect(m_updater, &StandardBackendUpdater::updatesCountChanged, this, &FwupdBackend::updatesCountChanged);
 
     SourcesModel::global()->addSourcesBackend(new FwupdSourcesBackend(this));
@@ -119,10 +126,22 @@ void FwupdBackend::addUpdates()
                     FwupdRelease *release = (FwupdRelease *)g_ptr_array_index(rels, j);
                     if (!fwupd_release_get_description(release))
                         continue;
-                    longdescription += QStringLiteral("Version %1\n").arg(QString::fromUtf8(fwupd_release_get_version(release)));
-                    longdescription += QString::fromUtf8(fwupd_release_get_description(release)) + QLatin1Char('\n');
+                    if (rels->len > 1) {
+                        longdescription += QStringLiteral("Version %1\n").arg(QString::fromUtf8(fwupd_release_get_version(release)));
+                    }
+                    longdescription += QString::fromUtf8(fwupd_release_get_description(release));
+                    if (rels->len > 1) {
+                        longdescription += QLatin1Char('\n');
+                    }
                 }
                 res->setDescription(longdescription);
+
+                // Make sure to set the installed version of the current thing so
+                // they can both be shown in the update page UI
+                auto installedResource = m_resources[res->packageName()];
+                if (installedResource) {
+                    res->setInstalledVersion(installedResource->availableVersion());
+                }
                 addResource(res);
             }
         } else {
@@ -319,13 +338,6 @@ void FwupdBackend::checkForUpdates()
     if (m_fetching)
         return;
 
-    g_autoptr(GError) error = nullptr;
-
-    if (!fwupd_client_connect(client, m_cancellable, &error)) {
-        handleError(error);
-        return;
-    }
-
     m_fetching = true;
     Q_EMIT fetchingChanged();
 
