@@ -7,28 +7,37 @@
 
 #include "FwupdResource.h"
 
-
-#include <Transaction/AddonList.h>
 #include <QDesktopServices>
 #include <QStringList>
 #include <QTimer>
+#include <Transaction/AddonList.h>
 
-FwupdResource::FwupdResource(QString name, AbstractResourcesBackend* parent)
+FwupdResource::FwupdResource(FwupdDevice *device, AbstractResourcesBackend *parent)
+    : FwupdResource(device,
+                    QStringLiteral("org.fwupd.%1.device").arg(QString::fromUtf8(fwupd_device_get_id(device)).replace(QLatin1Char('/'), QLatin1Char('_'))),
+                    parent)
+{
+}
+
+FwupdResource::FwupdResource(FwupdDevice *device, const QString &id, AbstractResourcesBackend *parent)
     : AbstractResource(parent)
-    , m_name(std::move(name))
+    , m_id(id)
+    , m_name(QString::fromUtf8(fwupd_device_get_name(device)))
+    , m_deviceID(QString::fromUtf8(fwupd_device_get_id(device)))
 {
     Q_ASSERT(!m_name.isEmpty());
     setObjectName(m_name);
+    setDeviceDetails(device);
 }
 
 QString FwupdResource::availableVersion() const
 {
-    return m_version;
+    return m_availableVersion;
 }
 
 QStringList FwupdResource::categories()
 {
-   return m_categories;
+    return m_categories;
 }
 
 QString FwupdResource::comment()
@@ -36,7 +45,7 @@ QString FwupdResource::comment()
     return m_summary;
 }
 
-int FwupdResource::size()
+quint64 FwupdResource::size()
 {
     return m_size;
 }
@@ -68,12 +77,12 @@ QVariant FwupdResource::icon() const
 
 QString FwupdResource::installedVersion() const
 {
-    return m_version;
+    return m_installedVersion;
 }
 
 QJsonArray FwupdResource::licenses()
 {
-    return { QJsonObject{ {QStringLiteral("name"), m_license} } };
+    return {QJsonObject{{QStringLiteral("name"), m_license}}};
 }
 
 QString FwupdResource::longDescription()
@@ -116,17 +125,15 @@ void FwupdResource::fetchChangelog()
     QString log = longDescription();
     log.replace(QLatin1Char('\n'), QLatin1String("<br />"));
 
-    emit changelogFetched(log);
+    Q_EMIT changelogFetched(log);
 }
 
 void FwupdResource::setState(AbstractResource::State state)
 {
-    if(m_state != state)
-    {
+    if (m_state != state) {
         m_state = state;
-        emit stateChanged();
+        Q_EMIT stateChanged();
     }
-
 }
 
 void FwupdResource::invokeApplication() const
@@ -144,44 +151,41 @@ QString FwupdResource::executeLabel() const
     return QStringLiteral("Not Invokable");
 }
 
-void FwupdResource::setReleaseDetails(FwupdRelease* release)
+void FwupdResource::setReleaseDetails(FwupdRelease *release)
 {
     m_origin = QString::fromUtf8(fwupd_release_get_remote_id(release));
     m_summary = QString::fromUtf8(fwupd_release_get_summary(release));
     m_vendor = QString::fromUtf8(fwupd_release_get_vendor(release));
     m_size = fwupd_release_get_size(release);
-    m_version = QString::fromUtf8(fwupd_release_get_version(release));
+    m_availableVersion = QString::fromUtf8(fwupd_release_get_version(release));
     m_description = QString::fromUtf8((fwupd_release_get_description(release)));
     m_homepage = QUrl(QString::fromUtf8(fwupd_release_get_homepage(release)));
     m_license = QString::fromUtf8(fwupd_release_get_license(release));
     m_updateURI = QString::fromUtf8(fwupd_release_get_uri(release));
 }
 
-void FwupdResource::setDeviceDetails(FwupdDevice* dev)
+void FwupdResource::setDeviceDetails(FwupdDevice *dev)
 {
     m_isLiveUpdatable = fwupd_device_has_flag(dev, FWUPD_DEVICE_FLAG_UPDATABLE);
     m_isOnlyOffline = fwupd_device_has_flag(dev, FWUPD_DEVICE_FLAG_ONLY_OFFLINE);
     m_needsReboot = fwupd_device_has_flag(dev, FWUPD_DEVICE_FLAG_NEEDS_REBOOT);
-    m_isDeviceRemoval = !fwupd_device_has_flag(dev, FWUPD_DEVICE_FLAG_INTERNAL);
-    m_needsBootLoader = fwupd_device_has_flag(dev, FWUPD_DEVICE_FLAG_NEEDS_BOOTLOADER);
 
-    if (fwupd_device_get_name(dev))
-    {
+    if (fwupd_device_get_name(dev)) {
         QString vendorDesc = QString::fromUtf8(fwupd_device_get_name(dev));
         const QString vendorName = QString::fromUtf8(fwupd_device_get_vendor(dev));
 
         if (!vendorDesc.startsWith(vendorName))
             vendorDesc = vendorName + QLatin1Char(' ') + vendorDesc;
         m_displayName = vendorDesc;
-     }
+    }
     m_summary = QString::fromUtf8(fwupd_device_get_summary(dev));
     m_vendor = QString::fromUtf8(fwupd_device_get_vendor(dev));
     m_releaseDate = QDateTime::fromSecsSinceEpoch(fwupd_device_get_created(dev)).date();
-    m_version = QString::fromUtf8(fwupd_device_get_version(dev));
+    m_availableVersion = QString::fromUtf8(fwupd_device_get_version(dev));
     m_description = QString::fromUtf8((fwupd_device_get_description(dev)));
 
     if (fwupd_device_get_icons(dev)->len >= 1)
-        m_iconName = QString::fromUtf8((const gchar *)g_ptr_array_index(fwupd_device_get_icons(dev), 0));// Check whether given icon exists or not!
+        m_iconName = QString::fromUtf8((const gchar *)g_ptr_array_index(fwupd_device_get_icons(dev), 0)); // Check whether given icon exists or not!
     else
         m_iconName = QStringLiteral("device-notifier");
 }
