@@ -12,6 +12,9 @@
 #include "PKTransaction.h"
 #include "PackageKitSourcesBackend.h"
 #include "PackageKitUpdater.h"
+#include <AppStreamQt/release.h>
+#include <AppStreamQt/utils.h>
+#include <AppStreamQt/version.h>
 #include <appstream/AppStreamIntegration.h>
 #include <appstream/AppStreamUtils.h>
 #include <appstream/OdrsReviewsBackend.h>
@@ -20,9 +23,9 @@
 #include <resources/StandardBackendUpdater.h>
 
 #include <QDebug>
-#include <QFile>
 #include <QFileSystemWatcher>
 #include <QFutureWatcher>
+#include <QHash>
 #include <QMimeDatabase>
 #include <QProcess>
 #include <QStandardPaths>
@@ -39,9 +42,30 @@
 #include "config-paths.h"
 #include "libdiscover_backend_debug.h"
 #include "utils.h"
+#include <Category/Category.h>
 
 DISCOVER_BACKEND_PLUGIN(PackageKitBackend)
 
+bool operator==(const PackageOrAppId &a, const PackageOrAppId &b)
+{
+    return a.isPackageName == b.isPackageName && a.id == b.id;
+}
+
+uint qHash(const PackageOrAppId &id, uint seed)
+{
+    return qHash(id.id, seed) ^ qHash(id.isPackageName, seed);
+}
+
+PackageOrAppId makeAppId(const QString &id)
+{
+    return {id, false};
+}
+
+PackageOrAppId makePackageId(const QString &id)
+{
+    return {id, true};
+}
+
 template<typename T, typename W>
 static void setWhenAvailable(const QDBusPendingReply<T> &pending, W func, QObject *parent)
 {
@@ -58,6 +82,17 @@ QString PackageKitBackend::locateService(const QString &filename)
     return QStandardPaths::locate(QStandardPaths::GenericDataLocation, QStringLiteral("applications/") + filename);
 }
 
+Delay::Delay()
+{
+    m_delay.setSingleShot(true);
+    m_delay.setInterval(100);
+
+    connect(&m_delay, &QTimer::timeout, this, [this] {
+        Q_EMIT perform(m_pkgids);
+        m_pkgids.clear();
+    });
+}
+
 PackageKitBackend::PackageKitBackend(QObject *parent)
     : AbstractResourcesBackend(parent)
     , m_appdata(new AppStream::Pool)
@@ -72,9 +107,56 @@ PackageKitBackend::PackageKitBackend(QObject *parent)
     t->setSingleShot(false);
     t->start();
 
-    m_delayedDetailsFetch.setSingleShot(true);
-    m_delayedDetailsFetch.setInterval(100);
-    connect(&m_delayedDetailsFetch, &QTimer::timeout, this, &PackageKitBackend::performDetailsFetch);
+    connect(&m_details, &Delay::perform, this, &PackageKitBackend::performDetailsFetch);
+    connect(&m_details, &Delay::perform, this, [this](const QSet<QString> &pkgids) {
+        PackageKit::Transaction *t = PackageKit::Daemon::getUpdatesDetails(kSetToList(pkgids));
+        connect(t,
+                &PackageKit::Transaction::updateDetail,
+                this,
+                [this](const QString &packageID,
+                       const QStringList &updates,
+                       const QStringList &obsoletes,
+                       const QStringList &vendorUrls,
+                       const QStringList &bugzillaUrls,
+                       const QStringList &cveUrls,
+                       PackageKit::Transaction::Restart restart,
+                       const QString &updateText,
+                       const QString &changelog,
+                       PackageKit::Transaction::UpdateState state,
+                       const QDateTime &issued,
+                       const QDateTime &updated) {
+                    const QSet<AbstractResource *> resources = resourcesByPackageName(PackageKit::Daemon::packageName(packageID));
+                    for (auto r : resources) {
+                        PackageKitResource *resource = qobject_cast<PackageKitResource *>(r);
+                        if (resource->containsPackageId(packageID)) {
+                            resource->updateDetail(packageID,
+                                                   updates,
+                                                   obsoletes,
+                                                   vendorUrls,
+                                                   bugzillaUrls,
+                                                   cveUrls,
+                                                   restart,
+                                                   updateText,
+                                                   changelog,
+                                                   state,
+                                                   issued,
+                                                   updated);
+                        }
+                    }
+                });
+        connect(t, &PackageKit::Transaction::errorCode, this, [this, pkgids](PackageKit::Transaction::Error err, const QString &error) {
+            qWarning() << "error fetching updates:" << err << error;
+            for (const QString &pkgid : pkgids) {
+                const QSet<AbstractResource *> resources = resourcesByPackageName(PackageKit::Daemon::packageName(pkgid));
+                for (auto r : resources) {
+                    PackageKitResource *resource = qobject_cast<PackageKitResource *>(r);
+                    if (resource->containsPackageId(pkgid)) {
+                        Q_EMIT resource->changelogFetched(QString());
+                    }
+                }
+            }
+        });
+    });
 
     connect(PackageKit::Daemon::global(), &PackageKit::Daemon::restartScheduled, m_updater, &PackageKitUpdater::enableNeedsReboot);
     connect(PackageKit::Daemon::global(), &PackageKit::Daemon::isRunningChanged, this, &PackageKitBackend::checkDaemonRunning);
@@ -225,6 +307,32 @@ void PackageKitBackend::reloadPackageList()
             Q_EMIT loadedAppStream();
         }
         acquireFetching(false);
+
+        const QList<AppStream::Component> distroComponents = m_appdata->componentsById(AppStream::Utils::currentDistroComponentId());
+        if (distroComponents.isEmpty()) {
+            qWarning() << "no component found for" << AppStream::Utils::currentDistroComponentId();
+        }
+        for (const AppStream::Component &dc : distroComponents) {
+            const auto releases = dc.releases();
+            for (const auto &r : releases) {
+                int cmp = AppStream::Utils::vercmpSimple(r.version(), AppStreamIntegration::global()->osRelease()->versionId());
+                if (cmp == 0) {
+                    // Ignore (likely) empty date_eol entries that are parsed as the UNIX Epoch
+                    if (r.timestampEol().isNull() || r.timestampEol().toSecsSinceEpoch() == 0) {
+                        continue;
+                    }
+                    if (r.timestampEol() < QDateTime::currentDateTime()) {
+                        const QString releaseDate = QLocale().toString(r.timestampEol());
+                        Q_EMIT inlineMessageChanged(
+                            QSharedPointer<InlineMessage>::create(InlineMessage::Warning,
+                                                                  QStringLiteral("dialog-warning"),
+                                                                  i18nc("%1 is the date as formatted by the locale",
+                                                                        "Your operating system ended support on %1. Consider upgrading to a supported version.",
+                                                                        releaseDate)));
+                    }
+                }
+            }
+        }
     });
     fw->setFuture(QtConcurrent::run(&m_threadPool, &loadAppStream, m_appdata.get()));
 }
@@ -235,7 +343,7 @@ AppPackageKitResource *PackageKitBackend::addComponent(const AppStream::Componen
     const QStringList pkgNames = component.packageNames();
     Q_ASSERT(!pkgNames.isEmpty());
 
-    auto &resPos = m_packages.packages[component.id()];
+    auto &resPos = m_packages.packages[makeAppId(component.id())];
     AppPackageKitResource *res = qobject_cast<AppPackageKitResource *>(resPos);
     if (!res) {
         res = new AppPackageKitResource(component, pkgNames.at(0), this);
@@ -254,8 +362,12 @@ AppPackageKitResource *PackageKitBackend::addComponent(const AppStream::Componen
     return res;
 }
 
-void PackageKitBackend::resolvePackages(const QStringList &packageNames)
+PKResolveTransaction *PackageKitBackend::resolvePackages(const QStringList &packageNames)
 {
+    if (packageNames.isEmpty()) {
+        return nullptr;
+    }
+
     if (!m_resolveTransaction) {
         m_resolveTransaction = new PKResolveTransaction(this);
         connect(m_resolveTransaction, &PKResolveTransaction::allFinished, this, &PackageKitBackend::getPackagesFinished);
@@ -265,6 +377,7 @@ void PackageKitBackend::resolvePackages(const QStringList &packageNames)
     }
 
     m_resolveTransaction->addPackageNames(packageNames);
+    return m_resolveTransaction;
 }
 
 void PackageKitBackend::fetchUpdates()
@@ -326,12 +439,12 @@ void PackageKitBackend::includePackagesToAdd()
 
     acquireFetching(true);
     for (PackageKitResource *res : qAsConst(m_packagesToAdd)) {
-        m_packages.packages[res->packageName()] = res;
+        m_packages.packages[makePackageId(res->packageName())] = res;
     }
     for (PackageKitResource *res : qAsConst(m_packagesToDelete)) {
         const auto pkgs = m_packages.packageToApp.value(res->packageName(), {res->packageName()});
         for (const auto &pkg : pkgs) {
-            auto res = m_packages.packages.take(pkg);
+            auto res = m_packages.packages.take(makePackageId(pkg));
             if (res) {
                 if (AppPackageKitResource *ares = qobject_cast<AppPackageKitResource *>(res)) {
                     const auto extends = res->extends();
@@ -368,20 +481,42 @@ void PackageKitBackend::packageDetails(const PackageKit::Details &details)
 
 QSet<AbstractResource *> PackageKitBackend::resourcesByPackageName(const QString &name) const
 {
-    return resourcesByPackageNames<QSet<AbstractResource *>>({name});
+    return resourcesByPackageNames<QSet<AbstractResource *>>(QStringList{name});
 }
 
-template<typename T>
-T PackageKitBackend::resourcesByPackageNames(const QStringList &pkgnames) const
+template<typename T, typename W>
+T PackageKitBackend::resourcesByAppNames(const W &appNames) const
+{
+    T ret;
+    ret.reserve(appNames.size());
+    for (const QString &name : appNames) {
+        AbstractResource *res = m_packages.packages.value(makeAppId(name));
+        if (res) {
+            ret += res;
+        }
+    }
+    return ret;
+}
+
+template<typename T, typename W>
+T PackageKitBackend::resourcesByPackageNames(const W &pkgnames) const
 {
     T ret;
     ret.reserve(pkgnames.size());
-    for (const QString &name : pkgnames) {
-        const QStringList names = m_packages.packageToApp.value(name, QStringList(name));
-        for (const QString &name : names) {
-            AbstractResource *res = m_packages.packages.value(name);
-            if (res)
+    for (const QString &pkg_name : pkgnames) {
+        const QStringList app_names = m_packages.packageToApp.value(pkg_name, QStringList());
+        if (app_names.isEmpty()) {
+            AbstractResource *res = m_packages.packages.value(makePackageId(pkg_name));
+            if (res) {
                 ret += res;
+            }
+        } else {
+            for (const QString &app_id : app_names) {
+                AbstractResource *res = m_packages.packages.value(makeAppId(app_id));
+                if (res) {
+                    ret += res;
+                }
+            }
         }
     }
     return ret;
@@ -412,7 +547,11 @@ void PackageKitBackend::checkForUpdates()
 QList<AppStream::Component> PackageKitBackend::componentsById(const QString &id) const
 {
     Q_ASSERT(m_appstreamInitialized);
-    return m_appdata->componentsById(id);
+    auto comps = m_appdata->componentsById(id);
+    if (comps.isEmpty()) {
+        comps = m_appdata->componentsByProvided(AppStream::Provided::KindId, id);
+    }
+    return comps;
 }
 
 static const auto needsResolveFilter = [](AbstractResource *res) {
@@ -433,22 +572,35 @@ public:
         , backend(backend)
     {
         QTimer::singleShot(0, this, [resources, this]() {
-            setResources(resources);
-            finish();
+            sendResources(resources);
         });
     }
 
-    void setResources(const QVector<AbstractResource *> &res)
+    void sendResources(const QVector<AbstractResource *> &res, bool waitForResolved = false)
     {
-        if (res.isEmpty())
+        if (res.isEmpty()) {
+            finish();
             return;
+        }
 
+        Q_ASSERT(res.size() == QSet(res.constBegin(), res.constEnd()).size());
         const auto toResolve = kFilter<QVector<AbstractResource *>>(res, needsResolveFilter);
-        if (!toResolve.isEmpty())
-            backend->resolvePackages(kTransform<QStringList>(toResolve, [](AbstractResource *res) {
+        if (!toResolve.isEmpty()) {
+            auto transaction = backend->resolvePackages(kTransform<QStringList>(toResolve, [](AbstractResource *res) {
                 return res->packageName();
             }));
+            if (waitForResolved) {
+                Q_ASSERT(transaction);
+                connect(transaction, &QObject::destroyed, this, [this, res] {
+                    Q_EMIT resourcesFound(res);
+                    finish();
+                });
+                return;
+            }
+        }
+
         Q_EMIT resourcesFound(res);
+        finish();
     }
 
 private:
@@ -465,8 +617,7 @@ ResultsStream *PackageKitBackend::search(const AbstractResourcesBackend::Filters
             const auto resources = kTransform<QVector<AbstractResource *>>(m_packages.extendedBy.value(filter.extends), [](AppPackageKitResource *a) {
                 return a;
             });
-            stream->setResources(resources);
-            stream->finish();
+            stream->sendResources(resources, filter.state != AbstractResource::Broken);
         };
         runWhenInitialized(f, stream);
         return stream;
@@ -513,33 +664,38 @@ ResultsStream *PackageKitBackend::search(const AbstractResourcesBackend::Filters
         };
         runWhenInitialized(f, stream);
         return stream;
-    } else if (filter.search.isEmpty()) {
+    } else if (filter.search.isEmpty() && !filter.category) {
         auto stream = new PKResultsStream(this, QStringLiteral("PackageKitStream-all"));
         auto f = [this, filter, stream] {
             auto resources = kFilter<QVector<AbstractResource *>>(m_packages.packages, [](AbstractResource *res) {
                 return res->type() != AbstractResource::Technical && !qobject_cast<PackageKitResource *>(res)->isCritical()
                     && !qobject_cast<PackageKitResource *>(res)->extendsItself();
             });
-            stream->setResources(resources);
-            stream->finish();
+            stream->sendResources(resources);
         };
         runWhenInitialized(f, stream);
         return stream;
     } else {
         auto stream = new PKResultsStream(this, QStringLiteral("PackageKitStream-search"));
         const auto f = [this, stream, filter]() {
-            const QList<AppStream::Component> components = m_appdata->search(filter.search);
-            const QStringList ids = kTransform<QStringList>(components, [](const AppStream::Component &comp) {
+            const auto components = !filter.search.isEmpty() ? m_appdata->search(filter.search)
+#if ASQ_CHECK_VERSION(0, 15, 6)
+                                  : filter.category          ? AppStreamUtils::componentsByCategories(m_appdata.get(),
+                                                                                                      filter.category,
+                                                                                                      AppStream::Bundle::KindUnknown)
+#endif
+                                                             : m_appdata->components();
+            const QSet<QString> ids = kTransform<QSet<QString>>(components, [](const AppStream::Component &comp) {
                 return comp.id();
             });
             if (!ids.isEmpty()) {
-                const auto resources =
-                    kFilter<QVector<AbstractResource *>>(resourcesByPackageNames<QVector<AbstractResource *>>(ids), [](AbstractResource *res) {
-                        return !qobject_cast<PackageKitResource *>(res)->extendsItself();
-                    });
-                stream->setResources(resources);
+                const auto resources = kFilter<QVector<AbstractResource *>>(resourcesByAppNames<QVector<AbstractResource *>>(ids), [](AbstractResource *res) {
+                    return !qobject_cast<PackageKitResource *>(res)->extendsItself();
+                });
+                stream->sendResources(resources, filter.state != AbstractResource::Broken);
+            } else {
+                stream->finish();
             }
-            stream->finish();
         };
         runWhenInitialized(f, stream);
         return stream;
@@ -568,50 +724,25 @@ PKResultsStream *PackageKitBackend::findResourceByPackageName(const QUrl &url)
             return new PKResultsStream(this, QStringLiteral("PackageKitStream-localpkg"), {new LocalFilePKResource(url, this)});
         }
     } else if (url.scheme() == QLatin1String("appstream")) {
-        static const QMap<QString, QString> deprecatedAppstreamIds = {
-            {QStringLiteral("org.kde.krita.desktop"), QStringLiteral("krita.desktop")},
-            {QStringLiteral("org.kde.digikam.desktop"), QStringLiteral("digikam.desktop")},
-            {QStringLiteral("org.kde.ktorrent.desktop"), QStringLiteral("ktorrent.desktop")},
-            {QStringLiteral("org.kde.gcompris.desktop"), QStringLiteral("gcompris.desktop")},
-            {QStringLiteral("org.kde.kmymoney.desktop"), QStringLiteral("kmymoney.desktop")},
-            {QStringLiteral("org.kde.kolourpaint.desktop"), QStringLiteral("kolourpaint.desktop")},
-            {QStringLiteral("org.blender.blender.desktop"), QStringLiteral("blender.desktop")},
-        };
-
         const auto appstreamIds = AppStreamUtils::appstreamIds(url);
         if (appstreamIds.isEmpty())
             Q_EMIT passiveMessage(i18n("Malformed appstream url '%1'", url.toDisplayString()));
         else {
             auto stream = new PKResultsStream(this, QStringLiteral("PackageKitStream-appstream-url"));
             const auto f = [this, appstreamIds, stream]() {
-                AbstractResource *pkg = nullptr;
-
-                QStringList allAppStreamIds = appstreamIds;
-                {
-                    auto it = deprecatedAppstreamIds.constFind(appstreamIds.first());
-                    if (it != deprecatedAppstreamIds.constEnd()) {
-                        allAppStreamIds << *it;
+                auto toSend = QSet<AbstractResource *>();
+                toSend.reserve(appstreamIds.size());
+                for (const auto &appstreamId : appstreamIds) {
+                    const auto comps = componentsById(appstreamId);
+                    if (comps.isEmpty()) {
+                        continue;
+                    }
+                    auto resources = resourcesByComponents<QVector<AbstractResource *>>(comps);
+                    for (const auto &r : resources) {
+                        toSend.insert(r);
                     }
                 }
-
-                for (auto it = m_packages.packages.constBegin(), itEnd = m_packages.packages.constEnd(); it != itEnd; ++it) {
-                    const bool matches = kContains(allAppStreamIds, [&it](const QString &id) {
-                        static const QLatin1String desktopPostfix(".desktop");
-                        return it.key().compare(id, Qt::CaseInsensitive) == 0 ||
-                            // doing (id == id.key()+".desktop") without allocating
-                            (id.size() == (desktopPostfix.size() + it.key().size()) && id.endsWith(desktopPostfix)
-                             && id.startsWith(it.key(), Qt::CaseInsensitive));
-                    });
-                    if (matches) {
-                        pkg = it.value();
-                        break;
-                    }
-                }
-                if (pkg)
-                    stream->setResources({pkg});
-                stream->finish();
-                //         if (!pkg)
-                //             qCDebug(LIBDISCOVER_BACKEND_LOG) << "could not find" << host << deprecatedHost;
+                stream->sendResources(QVector(toSend.constBegin(), toSend.constEnd()));
             };
             runWhenInitialized(f, stream);
             return stream;
@@ -620,6 +751,23 @@ PKResultsStream *PackageKitBackend::findResourceByPackageName(const QUrl &url)
     return new PKResultsStream(this, QStringLiteral("PackageKitStream-unknown-url"), {});
 }
 
+template<typename T>
+T PackageKitBackend::resourcesByComponents(const QList<AppStream::Component> &comps) const
+{
+    T ret;
+    ret.reserve(comps.size());
+    QSet<QString> done;
+    for (const auto &comp : comps) {
+        if (comp.packageNames().isEmpty() || done.contains(comp.id())) {
+            continue;
+        }
+        done += comp.id();
+        ret << m_packages.packages.value(makeAppId(comp.id()));
+        Q_ASSERT(ret.constLast());
+    }
+    return ret;
+}
+
 bool PackageKitBackend::hasSecurityUpdates() const
 {
     return m_hasSecurityUpdates;
@@ -739,50 +887,49 @@ void PackageKitBackend::getUpdatesFinished(PackageKit::Transaction::Exit, uint)
         Q_EMIT updatesCountChanged();
 }
 
-bool PackageKitBackend::isPackageNameUpgradeable(const PackageKitResource *res) const
-{
-    return !upgradeablePackageId(res).isEmpty();
-}
-
 // Copy of Transaction::packageName that doesn't create a copy but just pass a reference
 // It's an optimisation as there's a bunch of allocations that happen from packageName
 // Having packageName return a QStringRef or a QStringView would fix this issue.
 // TODO Qt 6: Have packageName and similars return a QStringView
-static QStringRef TransactionpackageName(const QString &packageID)
+static QStringView TransactionpackageName(const QString &packageID)
 {
-    QStringRef ret;
-    ret = packageID.leftRef(packageID.indexOf(QLatin1Char(';')));
-    return ret;
+    return QStringView(packageID).left(packageID.indexOf(QLatin1Char(';')));
 }
 
-QString PackageKitBackend::upgradeablePackageId(const PackageKitResource *res) const
+bool PackageKitBackend::isPackageNameUpgradeable(const PackageKitResource *res) const
 {
     const QString name = res->packageName();
     for (const QString &pkgid : m_updatesPackageId) {
         if (TransactionpackageName(pkgid) == name)
-            return pkgid;
+            return true;
     }
-    return QString();
+    return false;
+}
+
+QSet<QString> PackageKitBackend::upgradeablePackageId(const PackageKitResource *res) const
+{
+    QSet<QString> ids;
+    const QString name = res->packageName();
+    for (const QString &pkgid : m_updatesPackageId) {
+        if (TransactionpackageName(pkgid) == name)
+            ids.insert(pkgid);
+    }
+    return ids;
 }
 
 void PackageKitBackend::fetchDetails(const QSet<QString> &pkgid)
 {
-    if (!m_delayedDetailsFetch.isActive()) {
-        m_delayedDetailsFetch.start();
-    }
-
-    m_packageNamesToFetchDetails += pkgid;
+    m_details.add(pkgid);
 }
 
-void PackageKitBackend::performDetailsFetch()
+void PackageKitBackend::performDetailsFetch(const QSet<QString> &pkgids)
 {
-    Q_ASSERT(!m_packageNamesToFetchDetails.isEmpty());
-    const auto ids = m_packageNamesToFetchDetails.values();
+    Q_ASSERT(!pkgids.isEmpty());
+    const auto ids = pkgids.values();
 
     PackageKit::Transaction *transaction = PackageKit::Daemon::getDetails(ids);
     connect(transaction, &PackageKit::Transaction::details, this, &PackageKitBackend::packageDetails);
     connect(transaction, &PackageKit::Transaction::errorCode, this, &PackageKitBackend::transactionError);
-    m_packageNamesToFetchDetails.clear();
 }
 
 void PackageKitBackend::checkDaemonRunning()
@@ -827,11 +974,11 @@ int PackageKitBackend::fetchingUpdatesProgress() const
     return percentage;
 }
 
-HelpfulError *PackageKitBackend::explainDysfunction() const
+InlineMessage *PackageKitBackend::explainDysfunction() const
 {
     const auto error = m_appdata->lastError();
     if (!error.isEmpty()) {
-        return new HelpfulError(QStringLiteral("network-disconnect"), error);
+        return new InlineMessage(InlineMessage::Error, QStringLiteral("network-disconnect"), error);
     }
     return AbstractResourcesBackend::explainDysfunction();
 }
