@@ -13,10 +13,13 @@
 #include <QSet>
 
 #include <KConfigGroup>
+#include <KFormat>
+#include <KIO/FileSystemFreeSpaceJob>
 #include <KLocalizedString>
 #include <KSharedConfig>
 
 #include "libdiscover_backend_debug.h"
+#include "pk-offline-private.h"
 #include "utils.h"
 
 int percentageWithStatus(PackageKit::Transaction::Status status, uint percentage)
@@ -57,14 +60,20 @@ static void kRemoveDuplicates(QJsonArray &input, std::function<QString(const QJs
 
 class SystemUpgrade : public AbstractResource
 {
+    Q_OBJECT
 public:
     SystemUpgrade(PackageKitBackend *backend)
         : AbstractResource(backend)
         , m_backend(backend)
+        , m_updateSizeTimer(new QTimer(this))
     {
         connect(m_backend, &AbstractResourcesBackend::resourceRemoved, this, [this](AbstractResource *res) {
             m_resources.remove(res);
         });
+
+        m_updateSizeTimer->setInterval(100);
+        m_updateSizeTimer->setSingleShot(true);
+        connect(m_updateSizeTimer, &QTimer::timeout, this, &SystemUpgrade::refreshResource);
     }
 
     QString packageName() const override
@@ -146,10 +155,6 @@ public:
     {
         return {};
     }
-    QString longDescription() override
-    {
-        return {};
-    }
     QString origin() const override
     {
         return {};
@@ -164,19 +169,32 @@ public:
     }
     QString upgradeText() const override
     {
-        return i18np("1 package to upgrade", "%1 packages to upgrade", withoutDuplicates().count());
+        return i18np("1 package will be upgraded", "%1 packages will be upgraded", withoutDuplicates().count());
     }
-    void fetchChangelog() override
+    QString longDescription() override
     {
         QStringList changes;
         const auto resources = withoutDuplicates();
         for (auto res : resources) {
-            const auto versions = res->upgradeText();
-            const auto idx = versions.indexOf(u'\u009C');
-            changes += QStringLiteral("<li>") + res->packageName() + QStringLiteral(": ") + versions.leftRef(idx) + QStringLiteral("</li>\n");
+            const auto changelog = res->changelog();
+            if (changelog.isEmpty()) {
+                changes += i18n("<h3>%1</h3>Upgrade to new version %2<br/>No release notes provided", res->packageName(), res->availableVersion());
+            } else {
+                changes += i18n("<h3>%1</h3>Upgrade to new version %2<br/>Release notes:<blockquote>%3</blockquote>",
+                                res->packageName(),
+                                res->availableVersion(),
+                                changelog);
+            }
         }
         changes.sort();
-        Q_EMIT changelogFetched(QStringLiteral("<ul>") + changes.join(QString()) + QStringLiteral("</ul>\n"));
+        return changes.join(QString());
+    }
+    void fetchChangelog() override
+    {
+        for (auto res : qAsConst(m_resources)) {
+            res->fetchUpdateDetails();
+        }
+        Q_EMIT changelogFetched({});
     }
 
     QString installedVersion() const override
@@ -213,24 +231,39 @@ public:
     void refreshResource()
     {
         Q_EMIT m_backend->resourcesChanged(this, {"size", "license"});
+        Q_EMIT updateSizeChanged();
     }
 
     void setCandidates(const QSet<AbstractResource *> &candidates)
     {
-        for (auto res : (m_resources - candidates)) {
-            disconnect(res, &AbstractResource::sizeChanged, this, &SystemUpgrade::refreshResource);
+        const auto toDisconnect = (m_resources - candidates);
+        for (auto res : toDisconnect) {
+            disconnect(res, &AbstractResource::sizeChanged, this, &SystemUpgrade::startIfStopped);
+            disconnect(res, &AbstractResource::changelogFetched, this, &SystemUpgrade::startIfStopped);
         }
 
         const auto newCandidates = (candidates - m_resources);
         m_resources = candidates;
         for (auto res : newCandidates) {
-            connect(res, &AbstractResource::sizeChanged, this, &SystemUpgrade::refreshResource);
+            connect(res, &AbstractResource::sizeChanged, this, &SystemUpgrade::startIfStopped);
+            connect(res, &AbstractResource::changelogFetched, this, &SystemUpgrade::startIfStopped);
         }
     }
 
+    void startIfStopped()
+    {
+        if (!m_updateSizeTimer->isActive()) {
+            m_updateSizeTimer->start();
+        }
+    }
+
+Q_SIGNALS:
+    void updateSizeChanged();
+
 private:
     QSet<AbstractResource *> m_resources;
     PackageKitBackend *const m_backend;
+    QTimer *m_updateSizeTimer;
 };
 
 PackageKitUpdater::PackageKitUpdater(PackageKitBackend *parent)
@@ -259,18 +292,38 @@ void PackageKitUpdater::prepare()
         return;
     }
 
+    if (QFile::exists(QStringLiteral(PK_OFFLINE_RESULTS_FILENAME))) {
+        qDebug() << "Removed offline results file";
+        PackageKit::Daemon::global()->offline()->clearResults();
+    }
+
     Q_ASSERT(!m_transaction);
     const auto candidates = m_backend->upgradeablePackages();
     if (useOfflineUpdates() && !candidates.isEmpty()) {
         m_upgrade->setCandidates(candidates);
 
         m_toUpgrade = {m_upgrade};
+        connect(m_upgrade, &SystemUpgrade::updateSizeChanged, this, &PackageKitUpdater::checkFreeSpace);
     } else {
         m_toUpgrade = candidates;
     }
+
+    checkFreeSpace();
     m_allUpgradeable = m_toUpgrade;
 }
 
+void PackageKitUpdater::checkFreeSpace()
+{
+    auto j = KIO::fileSystemFreeSpace(QUrl::fromLocalFile("/usr"));
+    connect(j, &KIO::FileSystemFreeSpaceJob::result, this, [this](KIO::Job * /*job*/, KIO::filesize_t /*size*/, KIO::filesize_t available) {
+        if (available < updateSize()) {
+            setErrorMessage(i18nc("@info:status %1 is a formatted disk space string e.g. '240 MiB'",
+                                  "Not enough space to perform the update; only %1 of space are available.",
+                                  KFormat().formatByteSize(available)));
+        }
+    });
+}
+
 void PackageKitUpdater::setupTransaction(PackageKit::Transaction::TransactionFlags flags)
 {
     m_packagesModified.clear();
@@ -438,7 +491,7 @@ void PackageKitUpdater::finished(PackageKit::Transaction::Exit exit, uint /*time
 
         if (!toremove.isEmpty()) {
             QStringList criticals;
-            for (const auto &pkgid : toremove) {
+            for (const auto &pkgid : std::as_const(toremove)) {
                 auto res = kFilter<QVector<AbstractResource *>>(m_backend->resourcesByPackageName(pkgid), [](AbstractResource *res) {
                     return static_cast<PackageKitResource *>(res)->isCritical();
                 });
@@ -558,7 +611,7 @@ void PackageKitUpdater::errorFound(PackageKit::Transaction::Error err, const QSt
         || err == PackageKit::Transaction::ErrorNotAuthorized) {
         return;
     }
-    QString finalMessage = xi18nc("@info", "%1:<nl/><nl/>%2", PackageKitMessages::errorMessage(err), error);
+    QString finalMessage = xi18nc("@info", "%1:<nl/><nl/>%2", PackageKitMessages::errorMessage(err, QString()), error);
     Q_EMIT passiveMessage(finalMessage);
     qWarning() << "Error happened" << err << error;
 }
@@ -753,3 +806,5 @@ quint64 PackageKitUpdater::downloadSpeed() const
 {
     return m_transaction ? m_transaction->speed() : 0;
 }
+
+#include "PackageKitUpdater.moc"
