@@ -12,6 +12,7 @@
 
 #include <AppStreamQt/release.h>
 #include <AppStreamQt/utils.h>
+#include <Category/Category.h>
 #include <KLocalizedString>
 #include <appstream/AppStreamIntegration.h>
 
@@ -474,6 +475,14 @@ bool RpmOstreeBackend::isValid() const
 
 ResultsStream *RpmOstreeBackend::search(const AbstractResourcesBackend::Filters &filter)
 {
+    // Only return results when we're explicitely looking in the "Operating System" category
+    if (!filter.category) {
+        return new ResultsStream(QStringLiteral("rpm-ostree-empty"), {});
+    }
+    if (filter.category->untranslatedName() != QLatin1String("Operating System")) {
+        return new ResultsStream(QStringLiteral("rpm-ostree-empty"), {});
+    }
+
     QVector<AbstractResource *> res;
     for (RpmOstreeResource *r : m_resources) {
         if (r->state() >= filter.state) {
