@@ -4,8 +4,7 @@
  *   SPDX-License-Identifier: GPL-2.0-only OR GPL-3.0-only OR LicenseRef-KDE-Accepted-GPL
  */
 
-#ifndef ABSTRACTBACKENDUPDATER_H
-#define ABSTRACTBACKENDUPDATER_H
+#pragma once
 
 #include "discovercommon_export.h"
 #include <QObject>
@@ -43,6 +42,7 @@ class DISCOVERCOMMON_EXPORT AbstractBackendUpdater : public QObject
     Q_PROPERTY(bool isProgressing READ isProgressing NOTIFY progressingChanged)
     Q_PROPERTY(bool needsReboot READ needsReboot NOTIFY needsRebootChanged)
     Q_PROPERTY(quint64 downloadSpeed READ downloadSpeed NOTIFY downloadSpeedChanged)
+    Q_PROPERTY(QString errorMessage READ errorMessage NOTIFY errorMessageChanged)
 public:
     enum State {
         None,
@@ -50,7 +50,7 @@ public:
         Installing,
         Done,
     };
-    Q_ENUM(State);
+    Q_ENUM(State)
 
     /**
      * Constructs an AbstractBackendUpdater
@@ -58,7 +58,7 @@ public:
     explicit AbstractBackendUpdater(QObject *parent = nullptr);
 
     /**
-     * This method is called, when Muon switches to the updates view.
+     * This method is called, when Discover switches to the updates view.
      * Here the backend should mark all upgradeable packages as to be upgraded.
      */
     virtual void prepare() = 0;
@@ -159,6 +159,12 @@ public Q_SLOTS:
     {
     }
 
+    void setErrorMessage(const QString &errorMessage);
+    QString errorMessage() const
+    {
+        return m_errorMessage;
+    }
+
 Q_SIGNALS:
     /**
      * The AbstractBackendUpdater should Q_EMIT this signal when the progress changed.
@@ -218,9 +224,11 @@ Q_SIGNALS:
      */
     void needsRebootChanged();
 
+    /** emitted when we find a new errorMessage to display */
+    void errorMessageChanged();
+
 private:
     bool m_needsReboot = false;
     bool m_readyToReboot = false;
+    QString m_errorMessage;
 };
-
-#endif // ABSTRACTBACKENDUPDATER_H
