@@ -13,6 +13,7 @@
 #include <KShell>
 #include <QList>
 #include <QProcess>
+#include <QString>
 #include <ReviewsBackend/AbstractReviewsBackend.h>
 #include <ReviewsBackend/Rating.h>
 
@@ -46,6 +47,11 @@ QUrl AbstractResource::donationURL()
     return QUrl();
 }
 
+QUrl AbstractResource::contributeURL()
+{
+    return {};
+}
+
 void AbstractResource::addMetadata(const QString &key, const QJsonValue &value)
 {
     m_metadata.insert(key, value);
@@ -68,7 +74,7 @@ bool AbstractResource::isInstalled()
 
 void AbstractResource::fetchScreenshots()
 {
-    Q_EMIT screenshotsFetched({}, {});
+    Q_EMIT screenshotsFetched({});
 }
 
 QStringList AbstractResource::mimetypes() const
@@ -81,6 +87,11 @@ AbstractResourcesBackend *AbstractResource::backend() const
     return static_cast<AbstractResourcesBackend *>(parent());
 }
 
+QObject *AbstractResource::backendObject() const
+{
+    return parent();
+}
+
 QString AbstractResource::status()
 {
     switch (state()) {
@@ -140,62 +151,57 @@ void AbstractResource::reportNewState()
     Q_EMIT backend()->resourcesChanged(this, ns);
 }
 
-static bool shouldFilter(AbstractResource *res, const QPair<FilterType, QString> &filter)
+static bool shouldFilter(AbstractResource *res, const CategoryFilter &filter)
 {
     bool ret = true;
-    switch (filter.first) {
-    case CategoryFilter:
-        ret = res->categories().contains(filter.second);
+    switch (filter.type) {
+    case CategoryFilter::CategoryNameFilter:
+        ret = res->categories().contains(std::get<QString>(filter.value));
         break;
-    case PkgSectionFilter:
-        ret = res->section() == filter.second;
+    case CategoryFilter::PkgSectionFilter:
+        ret = res->section() == std::get<QString>(filter.value);
         break;
-    case PkgWildcardFilter: {
-        QString wildcard = filter.second;
+    case CategoryFilter::PkgWildcardFilter: {
+        QString wildcard = std::get<QString>(filter.value);
         wildcard.remove(QLatin1Char('*'));
         ret = res->packageName().contains(wildcard);
     } break;
-    case AppstreamIdWildcardFilter: {
-        QString wildcard = filter.second;
+    case CategoryFilter::AppstreamIdWildcardFilter: {
+        QString wildcard = std::get<QString>(filter.value);
         wildcard.remove(QLatin1Char('*'));
         ret = res->appstreamId().contains(wildcard);
     } break;
-    case PkgNameFilter: // Only useful in the not filters
-        ret = res->packageName() == filter.second;
+    case CategoryFilter::PkgNameFilter: // Only useful in the not filters
+        ret = res->packageName() == std::get<QString>(filter.value);
         break;
-    case InvalidFilter:
+    case CategoryFilter::AndFilter: {
+        const auto filters = std::get<QVector<CategoryFilter>>(filter.value);
+        ret = std::all_of(filters.begin(), filters.end(), [res](const CategoryFilter &f) {
+            return shouldFilter(res, f);
+        });
         break;
     }
+    case CategoryFilter::OrFilter: {
+        const auto filters = std::get<QVector<CategoryFilter>>(filter.value);
+        ret = std::any_of(filters.begin(), filters.end(), [res](const CategoryFilter &f) {
+            return shouldFilter(res, f);
+        });
+        break;
+    }
+    case CategoryFilter::NotFilter: {
+        const auto filters = std::get<QVector<CategoryFilter>>(filter.value);
+        ret = !std::any_of(filters.begin(), filters.end(), [res](const CategoryFilter &f) {
+            return shouldFilter(res, f);
+        });
+        break;
+    }
+    }
     return ret;
 }
 
 bool AbstractResource::categoryMatches(Category *cat)
 {
-    {
-        const auto orFilters = cat->orFilters();
-        bool orValue = orFilters.isEmpty();
-        for (const auto &filter : orFilters) {
-            if (shouldFilter(this, filter)) {
-                orValue = true;
-                break;
-            }
-        }
-        if (!orValue)
-            return false;
-    }
-
-    const auto andFilters = cat->andFilters();
-    for (const auto &filter : andFilters) {
-        if (!shouldFilter(this, filter))
-            return false;
-    }
-
-    const auto notFilters = cat->notFilters();
-    for (const auto &filter : notFilters) {
-        if (shouldFilter(this, filter))
-            return false;
-    }
-    return true;
+    return shouldFilter(this, cat->filter());
 }
 
 static QSet<Category *> walkCategories(AbstractResource *res, const QVector<Category *> &cats)
@@ -220,17 +226,6 @@ QSet<Category *> AbstractResource::categoryObjects(const QVector<Category *> &ca
     return walkCategories(const_cast<AbstractResource *>(this), cats);
 }
 
-QString AbstractResource::categoryDisplay() const
-{
-    const auto matchedCategories = categoryObjects(CategoryModel::global()->rootCategories());
-    QStringList ret;
-    for (auto cat : matchedCategories) {
-        ret.append(cat->name());
-    }
-    ret.sort();
-    return ret.join(QLatin1String(", "));
-}
-
 QUrl AbstractResource::url() const
 {
     const QString asid = appstreamId();
@@ -239,7 +234,7 @@ QUrl AbstractResource::url() const
 
 QString AbstractResource::displayOrigin() const
 {
-    return i18nc("origin (backend name)", "%1 (%2)", origin(), backend()->displayName());
+    return origin();
 }
 
 QString AbstractResource::executeLabel() const
@@ -253,7 +248,7 @@ QString AbstractResource::upgradeText() const
     if (installed == available) {
         // Update of the same version; show when old and new are
         // the same (common with Flatpak runtimes)
-        return i18n("Update to version %1", available);
+        return i18nc("@info 'Refresh' is used as a noun here, and %1 is an app's version number", "Refresh of version %1", available);
     } else if (!installed.isEmpty() && !available.isEmpty()) {
         // Old and new version numbers
         // This thing with \u009C is a fancy feature in QML text handling:
@@ -286,3 +281,23 @@ QString AbstractResource::versionString()
         }
     }
 }
+
+QString AbstractResource::contentRatingDescription() const
+{
+    return {};
+}
+
+AbstractResource::ContentIntensity AbstractResource::contentRatingIntensity() const
+{
+    return Mild;
+}
+
+QString AbstractResource::contentRatingText() const
+{
+    return {};
+}
+
+uint AbstractResource::contentRatingMinimumAge() const
+{
+    return 0;
+}
