@@ -32,27 +32,27 @@ ResourcesModel *ResourcesModel::s_self = nullptr;
 
 ResourcesModel *ResourcesModel::global()
 {
-    if (!s_self)
+    if (!s_self) {
         s_self = new ResourcesModel;
+        s_self->init(true);
+    }
     return s_self;
 }
 
-ResourcesModel::ResourcesModel(QObject *parent, bool load)
+ResourcesModel::ResourcesModel(QObject *parent)
     : QObject(parent)
     , m_isFetching(false)
-    , m_initializingBackends(0)
+    , m_initializingBackendsCount(0)
     , m_currentApplicationBackend(nullptr)
     , m_allInitializedEmitter(new QTimer(this))
     , m_updatesCount(
           0,
           [this] {
-              {
-                  int ret = 0;
-                  for (AbstractResourcesBackend *backend : qAsConst(m_backends)) {
-                      ret += backend->updatesCount();
-                  }
-                  return ret;
+              int ret = 0;
+              for (AbstractResourcesBackend *backend : qAsConst(m_backends)) {
+                  ret += backend->updatesCount();
               }
+              return ret;
           },
           [this](int count) {
               Q_EMIT updatesCountChanged(count);
@@ -60,40 +60,39 @@ ResourcesModel::ResourcesModel(QObject *parent, bool load)
     , m_fetchingUpdatesProgress(
           0,
           [this] {
-              {
-                  if (m_backends.isEmpty())
-                      return 0;
+              if (m_backends.isEmpty())
+                  return 0;
 
-                  int sum = 0;
-                  for (auto backend : qAsConst(m_backends)) {
-                      sum += backend->fetchingUpdatesProgress();
-                  }
-                  return sum / m_backends.count();
+              int sum = 0;
+              for (auto backend : qAsConst(m_backends)) {
+                  sum += backend->fetchingUpdatesProgress();
               }
+              return sum / (int)m_backends.count();
           },
           [this](int progress) {
               Q_EMIT fetchingUpdatesProgressChanged(progress);
           })
 {
-    init(load);
     connect(this, &ResourcesModel::allInitialized, this, &ResourcesModel::slotFetching);
     connect(this, &ResourcesModel::backendsChanged, this, &ResourcesModel::initApplicationsBackend);
 }
 
 void ResourcesModel::init(bool load)
 {
-    Q_ASSERT(!s_self);
     Q_ASSERT(QCoreApplication::instance()->thread() == QThread::currentThread());
 
     m_allInitializedEmitter->setSingleShot(true);
     m_allInitializedEmitter->setInterval(0);
     connect(m_allInitializedEmitter, &QTimer::timeout, this, [this]() {
-        if (m_initializingBackends == 0)
+        if (m_initializingBackendsCount == 0) {
+            m_isInitializing = false;
             Q_EMIT allInitialized();
+        }
     });
 
-    if (load)
-        QMetaObject::invokeMethod(this, "registerAllBackends", Qt::QueuedConnection);
+    if (load) {
+        registerAllBackends();
+    }
 
     m_updateAction = new DiscoverAction(this);
     m_updateAction->setIconName(QStringLiteral("system-software-update"));
@@ -108,10 +107,11 @@ void ResourcesModel::init(bool load)
 }
 
 ResourcesModel::ResourcesModel(const QString &backendName, QObject *parent)
-    : ResourcesModel(parent, false)
+    : ResourcesModel(parent)
 {
     s_self = this;
     registerBackendByName(backendName);
+    init(false);
 }
 
 ResourcesModel::~ResourcesModel()
@@ -134,7 +134,7 @@ void ResourcesModel::addResourcesBackend(AbstractResourcesBackend *backend)
     if (!backend->isFetching()) {
         m_updatesCount.reevaluate();
     } else {
-        m_initializingBackends++;
+        m_initializingBackendsCount++;
     }
 
     connect(backend, &AbstractResourcesBackend::fetchingChanged, this, &ResourcesModel::callerFetchingChanged);
@@ -148,6 +148,7 @@ void ResourcesModel::addResourcesBackend(AbstractResourcesBackend *backend)
     });
     connect(backend, &AbstractResourcesBackend::resourceRemoved, this, &ResourcesModel::resourceRemoved);
     connect(backend, &AbstractResourcesBackend::passiveMessage, this, &ResourcesModel::passiveMessage);
+    connect(backend, &AbstractResourcesBackend::inlineMessageChanged, this, &ResourcesModel::setInlineMessage);
     connect(backend->backendUpdater(), &AbstractBackendUpdater::progressingChanged, this, &ResourcesModel::slotFetching);
     if (backend->reviewsBackend()) {
         connect(backend->reviewsBackend(), &AbstractReviewsBackend::error, this, &ResourcesModel::passiveMessage, Qt::UniqueConnection);
@@ -157,7 +158,7 @@ void ResourcesModel::addResourcesBackend(AbstractResourcesBackend *backend)
     // pre-filled, we still need for the rest of the backends to be added before trying
     // to send out the initialized signal. To ensure this happens, schedule it for the
     // start of the next run of the event loop.
-    if (m_initializingBackends == 0) {
+    if (m_initializingBackendsCount == 0) {
         m_allInitializedEmitter->start();
     } else {
         slotFetching();
@@ -181,11 +182,11 @@ void ResourcesModel::callerFetchingChanged()
     }
 
     if (backend->isFetching()) {
-        m_initializingBackends++;
+        m_initializingBackendsCount++;
         slotFetching();
     } else {
-        m_initializingBackends--;
-        if (m_initializingBackends == 0)
+        m_initializingBackendsCount--;
+        if (m_initializingBackendsCount == 0)
             m_allInitializedEmitter->start();
         else
             slotFetching();
@@ -234,7 +235,7 @@ void ResourcesModel::registerAllBackends()
 {
     DiscoverBackendsFactory f;
     const auto backends = f.allBackends();
-    if (m_initializingBackends == 0 && backends.isEmpty()) {
+    if (m_initializingBackendsCount == 0 && backends.isEmpty()) {
         qCWarning(LIBDISCOVER_LOG) << "Couldn't find any backends";
         m_allInitializedEmitter->start();
     } else {
@@ -260,6 +261,11 @@ bool ResourcesModel::isFetching() const
     return m_isFetching;
 }
 
+bool ResourcesModel::isInitializing() const
+{
+    return m_isInitializing;
+}
+
 void ResourcesModel::slotFetching()
 {
     bool newFetching = false;
@@ -423,3 +429,13 @@ QUrl ResourcesModel::distroBugReportUrl()
 {
     return QUrl(KOSRelease().bugReportUrl());
 }
+
+void ResourcesModel::setInlineMessage(const QSharedPointer<InlineMessage> &inlineMessage)
+{
+    if (inlineMessage == m_inlineMessage) {
+        return;
+    }
+
+    m_inlineMessage = inlineMessage;
+    Q_EMIT inlineMessageChanged(inlineMessage);
+}
