@@ -9,40 +9,42 @@
 
 #include "libdiscover_debug.h"
 #include <QMetaProperty>
+#include <cmath>
 #include <utils.h>
 
 #include "ResourcesModel.h"
 #include <Category/CategoryModel.h>
+#include <KLocalizedString>
 #include <ReviewsBackend/Rating.h>
 #include <Transaction/TransactionModel.h>
 
+const QHash<int, QByteArray> ResourcesProxyModel::s_roles = {{NameRole, "name"},
+                                                             {IconRole, "icon"},
+                                                             {CommentRole, "comment"},
+                                                             {StateRole, "state"},
+                                                             {RatingRole, "rating"},
+                                                             {RatingPointsRole, "ratingPoints"},
+                                                             {RatingCountRole, "ratingCount"},
+                                                             {SortableRatingRole, "sortableRating"},
+                                                             {InstalledRole, "isInstalled"},
+                                                             {ApplicationRole, "application"},
+                                                             {OriginRole, "origin"},
+                                                             {DisplayOriginRole, "displayOrigin"},
+                                                             {CanUpgrade, "canUpgrade"},
+                                                             {PackageNameRole, "packageName"},
+                                                             {CategoryRole, "category"},
+                                                             {SectionRole, "section"},
+                                                             {MimeTypes, "mimetypes"},
+                                                             {LongDescriptionRole, "longDescription"},
+                                                             {SourceIconRole, "sourceIcon"},
+                                                             {SizeRole, "size"},
+                                                             {ReleaseDateRole, "releaseDate"}};
+
 ResourcesProxyModel::ResourcesProxyModel(QObject *parent)
     : QAbstractListModel(parent)
     , m_sortRole(NameRole)
     , m_sortOrder(Qt::AscendingOrder)
     , m_sortByRelevancy(false)
-    , m_roles({{NameRole, "name"},
-               {IconRole, "icon"},
-               {CommentRole, "comment"},
-               {StateRole, "state"},
-               {RatingRole, "rating"},
-               {RatingPointsRole, "ratingPoints"},
-               {RatingCountRole, "ratingCount"},
-               {SortableRatingRole, "sortableRating"},
-               {InstalledRole, "isInstalled"},
-               {ApplicationRole, "application"},
-               {OriginRole, "origin"},
-               {DisplayOriginRole, "displayOrigin"},
-               {CanUpgrade, "canUpgrade"},
-               {PackageNameRole, "packageName"},
-               {CategoryRole, "category"},
-               {CategoryDisplayRole, "categoryDisplay"},
-               {SectionRole, "section"},
-               {MimeTypes, "mimetypes"},
-               {LongDescriptionRole, "longDescription"},
-               {SourceIconRole, "sourceIcon"},
-               {SizeRole, "size"},
-               {ReleaseDateRole, "releaseDate"}})
     , m_currentStream(nullptr)
 {
     // new QAbstractItemModelTester(this, this);
@@ -55,6 +57,14 @@ ResourcesProxyModel::ResourcesProxyModel(QObject *parent)
     connect(this, &QAbstractItemModel::modelReset, this, &ResourcesProxyModel::countChanged);
     connect(this, &QAbstractItemModel::rowsInserted, this, &ResourcesProxyModel::countChanged);
     connect(this, &QAbstractItemModel::rowsRemoved, this, &ResourcesProxyModel::countChanged);
+
+    QTimer *roughCountTimer = new QTimer(this);
+    roughCountTimer->setInterval(10);
+    roughCountTimer->setSingleShot(true);
+
+    connect(roughCountTimer, &QTimer::timeout, this, &ResourcesProxyModel::roughCountChanged);
+    connect(this, &ResourcesProxyModel::countChanged, roughCountTimer, qOverload<>(&QTimer::start));
+    connect(this, &ResourcesProxyModel::busyChanged, roughCountTimer, qOverload<>(&QTimer::start));
 }
 
 void ResourcesProxyModel::componentComplete()
@@ -65,7 +75,7 @@ void ResourcesProxyModel::componentComplete()
 
 QHash<int, QByteArray> ResourcesProxyModel::roleNames() const
 {
-    return m_roles;
+    return s_roles;
 }
 
 void ResourcesProxyModel::setSortRole(Roles sortRole)
@@ -258,6 +268,32 @@ QString ResourcesProxyModel::originFilter() const
     return m_filters.origin;
 }
 
+QString ResourcesProxyModel::filteredCategoryName() const
+{
+    return m_categoryName;
+}
+
+void ResourcesProxyModel::setFilteredCategoryName(const QString &cat)
+{
+    if (cat == m_categoryName)
+        return;
+
+    m_categoryName = cat;
+
+    auto category = CategoryModel::global()->findCategoryByName(cat);
+    if (category) {
+        setFiltersFromCategory(category);
+    } else {
+        qDebug() << "looking up wrong category or too early" << m_categoryName;
+        auto f = [this, cat] {
+            auto category = CategoryModel::global()->findCategoryByName(cat);
+            setFiltersFromCategory(category);
+        };
+        auto one = new OneTimeAction(f, this);
+        connect(CategoryModel::global(), &CategoryModel::rootCategoriesChanged, one, &OneTimeAction::trigger);
+    }
+}
+
 void ResourcesProxyModel::setFiltersFromCategory(Category *category)
 {
     if (category == m_filters.category)
@@ -301,12 +337,16 @@ void ResourcesProxyModel::invalidateFilter()
         return;
     }
 
+    if (!m_categoryName.isEmpty() && m_filters.category == nullptr) {
+        return;
+    }
+
     if (m_currentStream) {
         qCWarning(LIBDISCOVER_LOG) << "last stream isn't over yet" << m_filters << this;
         delete m_currentStream;
     }
 
-    m_currentStream = ResourcesModel::global()->search(m_filters);
+    m_currentStream = m_filters.backend ? m_filters.backend->search(m_filters) : ResourcesModel::global()->search(m_filters);
     Q_EMIT busyChanged(true);
 
     if (!m_displayedResources.isEmpty()) {
@@ -315,8 +355,8 @@ void ResourcesProxyModel::invalidateFilter()
         endResetModel();
     }
 
-    connect(m_currentStream, &AggregatedResultsStream::resourcesFound, this, &ResourcesProxyModel::addResources);
-    connect(m_currentStream, &AggregatedResultsStream::finished, this, [this]() {
+    connect(m_currentStream, &ResultsStream::resourcesFound, this, &ResourcesProxyModel::addResources);
+    connect(m_currentStream, &ResultsStream::destroyed, this, [this]() {
         m_currentStream = nullptr;
         Q_EMIT busyChanged(false);
     });
@@ -350,7 +390,13 @@ bool ResourcesProxyModel::lessThan(AbstractResource *leftPackage, AbstractResour
     } else if (role == CanUpgrade) {
         ret = leftValue.toBool();
     } else {
+#if QT_VERSION < QT_VERSION_CHECK(6, 0, 0)
         ret = leftValue < rightValue;
+#else
+        const auto order = QVariant::compare(leftValue, rightValue);
+        Q_ASSERT(order != QPartialOrdering::Unordered);
+        return order == QPartialOrdering::Less;
+#endif
     }
     return ret != (order != Qt::AscendingOrder);
 }
@@ -437,6 +483,16 @@ void ResourcesProxyModel::setAllBackends(bool allBackends)
     m_filters.allBackends = allBackends;
 }
 
+AbstractResourcesBackend *ResourcesProxyModel::backendFilter() const
+{
+    return m_filters.backend;
+}
+
+void ResourcesProxyModel::setBackendFilter(AbstractResourcesBackend *filtered)
+{
+    m_filters.backend = filtered;
+}
+
 QVariant ResourcesProxyModel::data(const QModelIndex &index, int role) const
 {
     if (!index.isValid()) {
@@ -495,7 +551,9 @@ bool ResourcesProxyModel::isSorted(const QVector<AbstractResource *> &resources)
 {
     auto last = resources.constFirst();
     for (auto it = resources.constBegin() + 1, itEnd = resources.constEnd(); it != itEnd; ++it) {
-        if (!lessThan(last, *it)) {
+        auto v1 = roleToValue(last, m_sortRole), v2 = roleToValue(*it, m_sortRole);
+        if (!lessThan(last, *it) && v1 != v2) {
+            qDebug() << "faulty sort" << last->name() << (*it)->name() << last << (*it);
             return false;
         }
         last = *it;
@@ -505,6 +563,8 @@ bool ResourcesProxyModel::isSorted(const QVector<AbstractResource *> &resources)
 
 void ResourcesProxyModel::sortedInsertion(const QVector<AbstractResource *> &_res)
 {
+    Q_ASSERT(_res.size() == QSet(_res.constBegin(), _res.constEnd()).size());
+
     auto resources = _res;
     Q_ASSERT(!resources.isEmpty());
 
@@ -596,7 +656,7 @@ void ResourcesProxyModel::refreshBackend(AbstractResourcesBackend *backend, cons
         found = true;
     }
 
-    if (found && properties.contains(m_roles.value(m_sortRole))) {
+    if (found && properties.contains(s_roles.value(m_sortRole))) {
         invalidateSorting();
     }
 }
@@ -638,3 +698,21 @@ bool ResourcesProxyModel::sortByRelevancy() const
 {
     return m_sortByRelevancy;
 }
+
+QString ResourcesProxyModel::roughCount() const
+{
+    const int rows = rowCount();
+    if (isBusy()) {
+        // We return an empty string because it's evidently confusing
+        if (rows == 0) {
+            return {};
+        }
+
+        // We convert rows=1234 into round=1000
+        const int round = std::pow(10, std::floor(std::log10(rows)));
+        if (round >= 1) {
+            return i18nc("an approximation number, like 3000+", "%1+", (rows / round) * round);
+        }
+    }
+    return QString::number(rows);
+}
