@@ -4,24 +4,21 @@
  *   SPDX-License-Identifier: GPL-2.0-only OR GPL-3.0-only OR LicenseRef-KDE-Accepted-GPL
  */
 
-#ifndef STOREDRESULTSSTREAM_H
-#define STOREDRESULTSSTREAM_H
+#pragma once
 
 #include "ResourcesModel.h"
 
 class DISCOVERCOMMON_EXPORT StoredResultsStream : public AggregatedResultsStream
 {
-Q_OBJECT
+    Q_OBJECT
 public:
-    StoredResultsStream(const QSet<ResultsStream*>& streams);
+    StoredResultsStream(const QSet<ResultsStream *> &streams);
 
-    QVector<AbstractResource*> resources() const;
+    QVector<AbstractResource *> resources() const;
 
 Q_SIGNALS:
-    void finishedResources(const QVector<AbstractResource*>& resources);
+    void finishedResources(const QVector<AbstractResource *> &resources);
 
 private:
-    QVector<AbstractResource*> m_resources;
+    QVector<AbstractResource *> m_resources;
 };
-
-#endif
