@@ -14,7 +14,11 @@
 #include <QDBusMessage>
 #include <QDBusPendingCall>
 #include <QDebug>
+#if QT_VERSION < QT_VERSION_CHECK(6, 0, 0)
 #include <QNetworkConfigurationManager>
+#else
+#include <QNetworkInformation>
+#endif
 #include <QProcess>
 
 #include <KIO/ApplicationLauncherJob>
@@ -31,6 +35,12 @@ DiscoverNotifier::DiscoverNotifier(QObject *parent)
 {
     m_settings = new UpdatesSettings(this);
     m_settingsWatcher = KConfigWatcher::create(m_settings->sharedConfig());
+#if QT_VERSION >= QT_VERSION_CHECK(6, 3, 0)
+    QNetworkInformation::instance()->load(QNetworkInformation::Feature::Reachability | QNetworkInformation::Feature::TransportMedium);
+    connect(QNetworkInformation::instance(), &QNetworkInformation::reachabilityChanged, this, &DiscoverNotifier::stateChanged);
+    connect(QNetworkInformation::instance(), &QNetworkInformation::transportMediumChanged, this, &DiscoverNotifier::stateChanged);
+#endif
+
     refreshUnattended();
     connect(m_settingsWatcher.data(), &KConfigWatcher::configChanged, this, [this](const KConfigGroup &group, const QByteArrayList &names) {
         if (group.config()->name() == m_settings->config()->name() && group.name() == "Global" && names.contains("UseUnattendedUpdates")) {
@@ -60,7 +70,7 @@ DiscoverNotifier::DiscoverNotifier(QObject *parent)
     updateStatusNotifier();
 
     // Only fetch updates after the system is comfortably booted
-    QTimer::singleShot(20s, this, &DiscoverNotifier::recheckSystemUpdateNeeded);
+    QTimer::singleShot(0s, this, &DiscoverNotifier::recheckSystemUpdateNeeded);
 }
 
 DiscoverNotifier::~DiscoverNotifier() = default;
@@ -90,13 +100,46 @@ void DiscoverNotifier::showDiscoverUpdates(const QString &xdgActivationToken)
     }
 }
 
+bool DiscoverNotifier::notifyAboutUpdates() const
+{
+    if (state() != NormalUpdates && state() != SecurityUpdates) {
+        // it's not very helpful to notify that everything is in order
+        return false;
+    }
+
+    if (m_settings->requiredNotificationInterval() < 0) {
+        return false;
+    }
+
+    // To configure to a random value, execute:
+    // kwriteconfig5 --file PlasmaDiscoverUpdates --group Global --key RequiredNotificationInterval 3600
+    const QDateTime earliestNextNotificationTime = m_settings->lastNotificationTime().addSecs(m_settings->requiredNotificationInterval());
+    if (earliestNextNotificationTime.isValid() && earliestNextNotificationTime > QDateTime::currentDateTimeUtc()) {
+        return false;
+    }
+
+    m_settings->setLastNotificationTime(QDateTime::currentDateTimeUtc());
+    m_settings->save();
+
+    auto method = QDBusMessage::createMethodCall(QStringLiteral("org.kde.discover"),
+                                                 QStringLiteral("/"),
+                                                 QStringLiteral("org.freedesktop.DBus.Peer"),
+                                                 QStringLiteral("Ping"));
+    auto call = QDBusConnection::sessionBus().asyncCall(method);
+    call.waitForFinished();
+    if (call.isValid()) {
+        return false;
+    }
+    return true;
+}
+
 void DiscoverNotifier::showUpdatesNotification()
 {
     if (m_updatesAvailableNotification) {
         m_updatesAvailableNotification->close();
     }
-    if (state() != NormalUpdates && state() != SecurityUpdates) {
-        // it's not very helpful to notify that everything is in order
+
+    if (!notifyAboutUpdates()) {
         return;
     }
 
@@ -112,7 +155,7 @@ void DiscoverNotifier::showUpdatesNotification()
     m_updatesAvailableNotification->setDefaultAction(name);
     m_updatesAvailableNotification->setActions({name});
     connect(m_updatesAvailableNotification, QOverload<unsigned int>::of(&KNotification::activated), this, [this] {
-        showDiscoverUpdates({});
+        showDiscoverUpdates(m_updatesAvailableNotification->xdgActivationToken());
     });
 }
 
@@ -139,15 +182,38 @@ void DiscoverNotifier::updateStatusNotifier()
 }
 
 // we only want to do unattended updates when on an ethernet or wlan network
+#if QT_VERSION < QT_VERSION_CHECK(6, 0, 0)
 static bool isConnectionAdequate(const QNetworkConfiguration &network)
 {
     return (network.bearerType() == QNetworkConfiguration::BearerEthernet || network.bearerType() == QNetworkConfiguration::BearerWLAN);
 }
+#elif QT_VERSION >= QT_VERSION_CHECK(6, 3, 0)
+static bool isConnectionAdequate()
+{
+    const auto info = QNetworkInformation::instance();
+    if (info->supports(QNetworkInformation::Feature::Metered)) {
+        return !info->isMetered();
+    } else {
+        const auto transport = info->transportMedium();
+        return transport == QNetworkInformation::TransportMedium::Ethernet || transport == QNetworkInformation::TransportMedium::WiFi;
+    }
+}
+#endif
 
 void DiscoverNotifier::refreshUnattended()
 {
     m_settings->read();
-    const auto enabled = m_settings->useUnattendedUpdates() && m_manager->isOnline() && isConnectionAdequate(m_manager->defaultConfiguration());
+
+    if (!notifyAboutUpdates()) {
+        return;
+    }
+
+    const auto enabled = m_settings->useUnattendedUpdates()
+#if QT_VERSION < QT_VERSION_CHECK(6, 0, 0)
+        && m_manager->isOnline() && isConnectionAdequate(m_manager->defaultConfiguration());
+#elif QT_VERSION >= QT_VERSION_CHECK(6, 3, 0)
+        && QNetworkInformation::instance()->reachability() == QNetworkInformation::Reachability::Online && isConnectionAdequate();
+#endif
     if (bool(m_unattended) == enabled)
         return;
 
@@ -165,7 +231,11 @@ DiscoverNotifier::State DiscoverNotifier::state() const
         return RebootRequired;
     else if (m_isBusy)
         return Busy;
+#if QT_VERSION < QT_VERSION_CHECK(6, 0, 0)
     else if (m_manager && !m_manager->isOnline())
+#else
+    else if (QNetworkInformation::instance()->reachability() != QNetworkInformation::Reachability::Online)
+#endif
         return Offline;
     else if (m_hasSecurityUpdates)
         return SecurityUpdates;
@@ -215,6 +285,7 @@ QString DiscoverNotifier::message() const
 
 void DiscoverNotifier::recheckSystemUpdateNeeded()
 {
+#if QT_VERSION < QT_VERSION_CHECK(6, 0, 0)
     if (!m_manager) {
         m_manager = new QNetworkConfigurationManager(this);
         connect(m_manager, &QNetworkConfigurationManager::onlineStateChanged, this, &DiscoverNotifier::stateChanged);
@@ -222,6 +293,7 @@ void DiscoverNotifier::recheckSystemUpdateNeeded()
             Q_EMIT stateChanged();
         }
     }
+#endif
 
     for (BackendNotifierModule *module : qAsConst(m_backends))
         module->recheckSystemUpdateNeeded();
