@@ -26,15 +26,16 @@ int main(int argc, char **argv)
 {
     QApplication app(argc, argv);
     app.setOrganizationDomain(QStringLiteral("kde.org"));
+#if QT_VERSION < QT_VERSION_CHECK(6, 0, 0)
     app.setAttribute(Qt::AA_UseHighDpiPixmaps, true);
-
+#endif
     KCrash::setFlags(KCrash::AutoRestart);
 
     NotifierItem notifier;
     bool hide = false;
     KDBusService::StartupOptions startup = {};
     {
-        KAboutData about(QStringLiteral("DiscoverNotifier"),
+        KAboutData about(QStringLiteral("discover.notifier"),
                          i18n("Discover Notifier"),
                          version,
                          i18n("System update status notifier"),
@@ -43,7 +44,7 @@ int main(int argc, char **argv)
         about.addAuthor(QStringLiteral("Aleix Pol Gonzalez"), {}, QStringLiteral("aleixpol@kde.org"));
         about.setProductName("discover/discover");
         about.setProgramLogo(app.windowIcon());
-        about.setTranslator(i18ndc(nullptr, "NAME OF TRANSLATORS", "Your names"), i18ndc(nullptr, "EMAIL OF TRANSLATORS", "Your emails"));
+        about.setTranslator(i18nc("NAME OF TRANSLATORS", "Your names"), i18nc("EMAIL OF TRANSLATORS", "Your emails"));
 
         KAboutData::setApplicationData(about);
 
